/* Autogenerated */
/* curve description: p384 */
/* requested operations: (all) */
/* s = 0x1000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000 (from "2^384") */
/* c = [(340282366920938463463374607431768211456, 1), (79228162514264337593543950336, 1), (4294967296, -1), (1, 1)] (from "2^128,1;2^96,1;2^32,-1;1,1") */
/* machine_wordsize = 64 (from "64") */
/*                                                                    */
/* NOTE: In addition to the bounds specified above each function, all */
/*   functions synthesized for this Montgomery arithmetic require the */
/*   input to be strictly less than the prime modulus (s-c), and also */
/*   require the input to be in the unique saturated representation.  */
/*   All functions also ensure that these two properties are true of  */
/*   return values.                                                   */

#include <stdint.h>
typedef unsigned char fiat_p384_uint1;
typedef signed char fiat_p384_int1;
typedef signed __int128 fiat_p384_int128;
typedef unsigned __int128 fiat_p384_uint128;


/*
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffffffffffff]
 *   arg3: [0x0 ~> 0xffffffffffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffffffffffff]
 *   out2: [0x0 ~> 0x1]
 */
static void fiat_p384_addcarryx_u64(uint64_t* out1, fiat_p384_uint1* out2, fiat_p384_uint1 arg1, uint64_t arg2, uint64_t arg3) {
  fiat_p384_uint128 x1 = ((arg1 + (fiat_p384_uint128)arg2) + arg3);
  uint64_t x2 = (uint64_t)(x1 & UINT64_C(0xffffffffffffffff));
  fiat_p384_uint1 x3 = (fiat_p384_uint1)(x1 >> 64);
  *out1 = x2;
  *out2 = x3;
}

/*
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffffffffffff]
 *   arg3: [0x0 ~> 0xffffffffffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffffffffffff]
 *   out2: [0x0 ~> 0x1]
 */
static void fiat_p384_subborrowx_u64(uint64_t* out1, fiat_p384_uint1* out2, fiat_p384_uint1 arg1, uint64_t arg2, uint64_t arg3) {
  fiat_p384_int128 x1 = ((arg2 - (fiat_p384_int128)arg1) - arg3);
  fiat_p384_int1 x2 = (fiat_p384_int1)((fiat_p384_uint128)x1 >> 64);
  uint64_t x3 = (uint64_t)(x1 & UINT64_C(0xffffffffffffffff));
  *out1 = x3;
  *out2 = (fiat_p384_uint1)(0x0 - x2);
}

/*
 * Input Bounds:
 *   arg1: [0x0 ~> 0xffffffffffffffff]
 *   arg2: [0x0 ~> 0xffffffffffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffffffffffff]
 *   out2: [0x0 ~> 0xffffffffffffffff]
 */
static void fiat_p384_mulx_u64(uint64_t* out1, uint64_t* out2, uint64_t arg1, uint64_t arg2) {
  fiat_p384_uint128 x1 = ((fiat_p384_uint128)arg1 * arg2);
  uint64_t x2 = (uint64_t)(x1 & UINT64_C(0xffffffffffffffff));
  uint64_t x3 = (uint64_t)(x1 >> 64);
  *out1 = x2;
  *out2 = x3;
}

/*
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [0x0 ~> 0xffffffffffffffff]
 *   arg3: [0x0 ~> 0xffffffffffffffff]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffffffffffff]
 */
static void fiat_p384_cmovznz_u64(uint64_t* out1, fiat_p384_uint1 arg1, uint64_t arg2, uint64_t arg3) {
  fiat_p384_uint1 x1 = (!(!arg1));
  uint64_t x2 = ((fiat_p384_int1)(0x0 - x1) & UINT64_C(0xffffffffffffffff));
  uint64_t x3 = ((x2 & arg3) | ((~x2) & arg2));
  *out1 = x3;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 *   arg2: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_mul(uint64_t out1[6], const uint64_t arg1[6], const uint64_t arg2[6]) {
  uint64_t x1 = (arg1[1]);
  uint64_t x2 = (arg1[2]);
  uint64_t x3 = (arg1[3]);
  uint64_t x4 = (arg1[4]);
  uint64_t x5 = (arg1[5]);
  uint64_t x6 = (arg1[0]);
  uint64_t x7;
  uint64_t x8;
  fiat_p384_mulx_u64(&x7, &x8, x6, (arg2[5]));
  uint64_t x9;
  uint64_t x10;
  fiat_p384_mulx_u64(&x9, &x10, x6, (arg2[4]));
  uint64_t x11;
  uint64_t x12;
  fiat_p384_mulx_u64(&x11, &x12, x6, (arg2[3]));
  uint64_t x13;
  uint64_t x14;
  fiat_p384_mulx_u64(&x13, &x14, x6, (arg2[2]));
  uint64_t x15;
  uint64_t x16;
  fiat_p384_mulx_u64(&x15, &x16, x6, (arg2[1]));
  uint64_t x17;
  uint64_t x18;
  fiat_p384_mulx_u64(&x17, &x18, x6, (arg2[0]));
  uint64_t x19;
  fiat_p384_uint1 x20;
  fiat_p384_addcarryx_u64(&x19, &x20, 0x0, x15, x18);
  uint64_t x21;
  fiat_p384_uint1 x22;
  fiat_p384_addcarryx_u64(&x21, &x22, x20, x13, x16);
  uint64_t x23;
  fiat_p384_uint1 x24;
  fiat_p384_addcarryx_u64(&x23, &x24, x22, x11, x14);
  uint64_t x25;
  fiat_p384_uint1 x26;
  fiat_p384_addcarryx_u64(&x25, &x26, x24, x9, x12);
  uint64_t x27;
  fiat_p384_uint1 x28;
  fiat_p384_addcarryx_u64(&x27, &x28, x26, x7, x10);
  uint64_t x29;
  fiat_p384_uint1 x30;
  fiat_p384_addcarryx_u64(&x29, &x30, x28, 0x0, x8);
  uint64_t x31;
  uint64_t x32;
  fiat_p384_mulx_u64(&x31, &x32, x17, UINT64_C(0x100000001));
  uint64_t x33;
  uint64_t x34;
  fiat_p384_mulx_u64(&x33, &x34, x31, UINT64_C(0xffffffffffffffff));
  uint64_t x35;
  uint64_t x36;
  fiat_p384_mulx_u64(&x35, &x36, x31, UINT64_C(0xffffffffffffffff));
  uint64_t x37;
  uint64_t x38;
  fiat_p384_mulx_u64(&x37, &x38, x31, UINT64_C(0xffffffffffffffff));
  uint64_t x39;
  uint64_t x40;
  fiat_p384_mulx_u64(&x39, &x40, x31, UINT64_C(0xfffffffffffffffe));
  uint64_t x41;
  uint64_t x42;
  fiat_p384_mulx_u64(&x41, &x42, x31, UINT64_C(0xffffffff00000000));
  uint64_t x43;
  uint64_t x44;
  fiat_p384_mulx_u64(&x43, &x44, x31, UINT32_C(0xffffffff));
  uint64_t x45;
  fiat_p384_uint1 x46;
  fiat_p384_addcarryx_u64(&x45, &x46, 0x0, x41, x44);
  uint64_t x47;
  fiat_p384_uint1 x48;
  fiat_p384_addcarryx_u64(&x47, &x48, x46, x39, x42);
  uint64_t x49;
  fiat_p384_uint1 x50;
  fiat_p384_addcarryx_u64(&x49, &x50, x48, x37, x40);
  uint64_t x51;
  fiat_p384_uint1 x52;
  fiat_p384_addcarryx_u64(&x51, &x52, x50, x35, x38);
  uint64_t x53;
  fiat_p384_uint1 x54;
  fiat_p384_addcarryx_u64(&x53, &x54, x52, x33, x36);
  uint64_t x55;
  fiat_p384_uint1 x56;
  fiat_p384_addcarryx_u64(&x55, &x56, x54, 0x0, x34);
  uint64_t x57;
  fiat_p384_uint1 x58;
  fiat_p384_addcarryx_u64(&x57, &x58, 0x0, x43, x17);
  uint64_t x59;
  fiat_p384_uint1 x60;
  fiat_p384_addcarryx_u64(&x59, &x60, x58, x45, x19);
  uint64_t x61;
  fiat_p384_uint1 x62;
  fiat_p384_addcarryx_u64(&x61, &x62, x60, x47, x21);
  uint64_t x63;
  fiat_p384_uint1 x64;
  fiat_p384_addcarryx_u64(&x63, &x64, x62, x49, x23);
  uint64_t x65;
  fiat_p384_uint1 x66;
  fiat_p384_addcarryx_u64(&x65, &x66, x64, x51, x25);
  uint64_t x67;
  fiat_p384_uint1 x68;
  fiat_p384_addcarryx_u64(&x67, &x68, x66, x53, x27);
  uint64_t x69;
  fiat_p384_uint1 x70;
  fiat_p384_addcarryx_u64(&x69, &x70, x68, x55, x29);
  uint64_t x71;
  fiat_p384_uint1 x72;
  fiat_p384_addcarryx_u64(&x71, &x72, x70, 0x0, 0x0);
  uint64_t x73;
  uint64_t x74;
  fiat_p384_mulx_u64(&x73, &x74, x1, (arg2[5]));
  uint64_t x75;
  uint64_t x76;
  fiat_p384_mulx_u64(&x75, &x76, x1, (arg2[4]));
  uint64_t x77;
  uint64_t x78;
  fiat_p384_mulx_u64(&x77, &x78, x1, (arg2[3]));
  uint64_t x79;
  uint64_t x80;
  fiat_p384_mulx_u64(&x79, &x80, x1, (arg2[2]));
  uint64_t x81;
  uint64_t x82;
  fiat_p384_mulx_u64(&x81, &x82, x1, (arg2[1]));
  uint64_t x83;
  uint64_t x84;
  fiat_p384_mulx_u64(&x83, &x84, x1, (arg2[0]));
  uint64_t x85;
  fiat_p384_uint1 x86;
  fiat_p384_addcarryx_u64(&x85, &x86, 0x0, x81, x84);
  uint64_t x87;
  fiat_p384_uint1 x88;
  fiat_p384_addcarryx_u64(&x87, &x88, x86, x79, x82);
  uint64_t x89;
  fiat_p384_uint1 x90;
  fiat_p384_addcarryx_u64(&x89, &x90, x88, x77, x80);
  uint64_t x91;
  fiat_p384_uint1 x92;
  fiat_p384_addcarryx_u64(&x91, &x92, x90, x75, x78);
  uint64_t x93;
  fiat_p384_uint1 x94;
  fiat_p384_addcarryx_u64(&x93, &x94, x92, x73, x76);
  uint64_t x95;
  fiat_p384_uint1 x96;
  fiat_p384_addcarryx_u64(&x95, &x96, x94, 0x0, x74);
  uint64_t x97;
  fiat_p384_uint1 x98;
  fiat_p384_addcarryx_u64(&x97, &x98, 0x0, x83, x59);
  uint64_t x99;
  fiat_p384_uint1 x100;
  fiat_p384_addcarryx_u64(&x99, &x100, x98, x85, x61);
  uint64_t x101;
  fiat_p384_uint1 x102;
  fiat_p384_addcarryx_u64(&x101, &x102, x100, x87, x63);
  uint64_t x103;
  fiat_p384_uint1 x104;
  fiat_p384_addcarryx_u64(&x103, &x104, x102, x89, x65);
  uint64_t x105;
  fiat_p384_uint1 x106;
  fiat_p384_addcarryx_u64(&x105, &x106, x104, x91, x67);
  uint64_t x107;
  fiat_p384_uint1 x108;
  fiat_p384_addcarryx_u64(&x107, &x108, x106, x93, x69);
  uint64_t x109;
  fiat_p384_uint1 x110;
  fiat_p384_addcarryx_u64(&x109, &x110, x108, x95, (fiat_p384_uint1)x71);
  uint64_t x111;
  uint64_t x112;
  fiat_p384_mulx_u64(&x111, &x112, x97, UINT64_C(0x100000001));
  uint64_t x113;
  uint64_t x114;
  fiat_p384_mulx_u64(&x113, &x114, x111, UINT64_C(0xffffffffffffffff));
  uint64_t x115;
  uint64_t x116;
  fiat_p384_mulx_u64(&x115, &x116, x111, UINT64_C(0xffffffffffffffff));
  uint64_t x117;
  uint64_t x118;
  fiat_p384_mulx_u64(&x117, &x118, x111, UINT64_C(0xffffffffffffffff));
  uint64_t x119;
  uint64_t x120;
  fiat_p384_mulx_u64(&x119, &x120, x111, UINT64_C(0xfffffffffffffffe));
  uint64_t x121;
  uint64_t x122;
  fiat_p384_mulx_u64(&x121, &x122, x111, UINT64_C(0xffffffff00000000));
  uint64_t x123;
  uint64_t x124;
  fiat_p384_mulx_u64(&x123, &x124, x111, UINT32_C(0xffffffff));
  uint64_t x125;
  fiat_p384_uint1 x126;
  fiat_p384_addcarryx_u64(&x125, &x126, 0x0, x121, x124);
  uint64_t x127;
  fiat_p384_uint1 x128;
  fiat_p384_addcarryx_u64(&x127, &x128, x126, x119, x122);
  uint64_t x129;
  fiat_p384_uint1 x130;
  fiat_p384_addcarryx_u64(&x129, &x130, x128, x117, x120);
  uint64_t x131;
  fiat_p384_uint1 x132;
  fiat_p384_addcarryx_u64(&x131, &x132, x130, x115, x118);
  uint64_t x133;
  fiat_p384_uint1 x134;
  fiat_p384_addcarryx_u64(&x133, &x134, x132, x113, x116);
  uint64_t x135;
  fiat_p384_uint1 x136;
  fiat_p384_addcarryx_u64(&x135, &x136, x134, 0x0, x114);
  uint64_t x137;
  fiat_p384_uint1 x138;
  fiat_p384_addcarryx_u64(&x137, &x138, 0x0, x123, x97);
  uint64_t x139;
  fiat_p384_uint1 x140;
  fiat_p384_addcarryx_u64(&x139, &x140, x138, x125, x99);
  uint64_t x141;
  fiat_p384_uint1 x142;
  fiat_p384_addcarryx_u64(&x141, &x142, x140, x127, x101);
  uint64_t x143;
  fiat_p384_uint1 x144;
  fiat_p384_addcarryx_u64(&x143, &x144, x142, x129, x103);
  uint64_t x145;
  fiat_p384_uint1 x146;
  fiat_p384_addcarryx_u64(&x145, &x146, x144, x131, x105);
  uint64_t x147;
  fiat_p384_uint1 x148;
  fiat_p384_addcarryx_u64(&x147, &x148, x146, x133, x107);
  uint64_t x149;
  fiat_p384_uint1 x150;
  fiat_p384_addcarryx_u64(&x149, &x150, x148, x135, x109);
  uint64_t x151;
  fiat_p384_uint1 x152;
  fiat_p384_addcarryx_u64(&x151, &x152, x150, 0x0, x110);
  uint64_t x153;
  uint64_t x154;
  fiat_p384_mulx_u64(&x153, &x154, x2, (arg2[5]));
  uint64_t x155;
  uint64_t x156;
  fiat_p384_mulx_u64(&x155, &x156, x2, (arg2[4]));
  uint64_t x157;
  uint64_t x158;
  fiat_p384_mulx_u64(&x157, &x158, x2, (arg2[3]));
  uint64_t x159;
  uint64_t x160;
  fiat_p384_mulx_u64(&x159, &x160, x2, (arg2[2]));
  uint64_t x161;
  uint64_t x162;
  fiat_p384_mulx_u64(&x161, &x162, x2, (arg2[1]));
  uint64_t x163;
  uint64_t x164;
  fiat_p384_mulx_u64(&x163, &x164, x2, (arg2[0]));
  uint64_t x165;
  fiat_p384_uint1 x166;
  fiat_p384_addcarryx_u64(&x165, &x166, 0x0, x161, x164);
  uint64_t x167;
  fiat_p384_uint1 x168;
  fiat_p384_addcarryx_u64(&x167, &x168, x166, x159, x162);
  uint64_t x169;
  fiat_p384_uint1 x170;
  fiat_p384_addcarryx_u64(&x169, &x170, x168, x157, x160);
  uint64_t x171;
  fiat_p384_uint1 x172;
  fiat_p384_addcarryx_u64(&x171, &x172, x170, x155, x158);
  uint64_t x173;
  fiat_p384_uint1 x174;
  fiat_p384_addcarryx_u64(&x173, &x174, x172, x153, x156);
  uint64_t x175;
  fiat_p384_uint1 x176;
  fiat_p384_addcarryx_u64(&x175, &x176, x174, 0x0, x154);
  uint64_t x177;
  fiat_p384_uint1 x178;
  fiat_p384_addcarryx_u64(&x177, &x178, 0x0, x163, x139);
  uint64_t x179;
  fiat_p384_uint1 x180;
  fiat_p384_addcarryx_u64(&x179, &x180, x178, x165, x141);
  uint64_t x181;
  fiat_p384_uint1 x182;
  fiat_p384_addcarryx_u64(&x181, &x182, x180, x167, x143);
  uint64_t x183;
  fiat_p384_uint1 x184;
  fiat_p384_addcarryx_u64(&x183, &x184, x182, x169, x145);
  uint64_t x185;
  fiat_p384_uint1 x186;
  fiat_p384_addcarryx_u64(&x185, &x186, x184, x171, x147);
  uint64_t x187;
  fiat_p384_uint1 x188;
  fiat_p384_addcarryx_u64(&x187, &x188, x186, x173, x149);
  uint64_t x189;
  fiat_p384_uint1 x190;
  fiat_p384_addcarryx_u64(&x189, &x190, x188, x175, x151);
  uint64_t x191;
  uint64_t x192;
  fiat_p384_mulx_u64(&x191, &x192, x177, UINT64_C(0x100000001));
  uint64_t x193;
  uint64_t x194;
  fiat_p384_mulx_u64(&x193, &x194, x191, UINT64_C(0xffffffffffffffff));
  uint64_t x195;
  uint64_t x196;
  fiat_p384_mulx_u64(&x195, &x196, x191, UINT64_C(0xffffffffffffffff));
  uint64_t x197;
  uint64_t x198;
  fiat_p384_mulx_u64(&x197, &x198, x191, UINT64_C(0xffffffffffffffff));
  uint64_t x199;
  uint64_t x200;
  fiat_p384_mulx_u64(&x199, &x200, x191, UINT64_C(0xfffffffffffffffe));
  uint64_t x201;
  uint64_t x202;
  fiat_p384_mulx_u64(&x201, &x202, x191, UINT64_C(0xffffffff00000000));
  uint64_t x203;
  uint64_t x204;
  fiat_p384_mulx_u64(&x203, &x204, x191, UINT32_C(0xffffffff));
  uint64_t x205;
  fiat_p384_uint1 x206;
  fiat_p384_addcarryx_u64(&x205, &x206, 0x0, x201, x204);
  uint64_t x207;
  fiat_p384_uint1 x208;
  fiat_p384_addcarryx_u64(&x207, &x208, x206, x199, x202);
  uint64_t x209;
  fiat_p384_uint1 x210;
  fiat_p384_addcarryx_u64(&x209, &x210, x208, x197, x200);
  uint64_t x211;
  fiat_p384_uint1 x212;
  fiat_p384_addcarryx_u64(&x211, &x212, x210, x195, x198);
  uint64_t x213;
  fiat_p384_uint1 x214;
  fiat_p384_addcarryx_u64(&x213, &x214, x212, x193, x196);
  uint64_t x215;
  fiat_p384_uint1 x216;
  fiat_p384_addcarryx_u64(&x215, &x216, x214, 0x0, x194);
  uint64_t x217;
  fiat_p384_uint1 x218;
  fiat_p384_addcarryx_u64(&x217, &x218, 0x0, x203, x177);
  uint64_t x219;
  fiat_p384_uint1 x220;
  fiat_p384_addcarryx_u64(&x219, &x220, x218, x205, x179);
  uint64_t x221;
  fiat_p384_uint1 x222;
  fiat_p384_addcarryx_u64(&x221, &x222, x220, x207, x181);
  uint64_t x223;
  fiat_p384_uint1 x224;
  fiat_p384_addcarryx_u64(&x223, &x224, x222, x209, x183);
  uint64_t x225;
  fiat_p384_uint1 x226;
  fiat_p384_addcarryx_u64(&x225, &x226, x224, x211, x185);
  uint64_t x227;
  fiat_p384_uint1 x228;
  fiat_p384_addcarryx_u64(&x227, &x228, x226, x213, x187);
  uint64_t x229;
  fiat_p384_uint1 x230;
  fiat_p384_addcarryx_u64(&x229, &x230, x228, x215, x189);
  uint64_t x231;
  fiat_p384_uint1 x232;
  fiat_p384_addcarryx_u64(&x231, &x232, x230, 0x0, x190);
  uint64_t x233;
  uint64_t x234;
  fiat_p384_mulx_u64(&x233, &x234, x3, (arg2[5]));
  uint64_t x235;
  uint64_t x236;
  fiat_p384_mulx_u64(&x235, &x236, x3, (arg2[4]));
  uint64_t x237;
  uint64_t x238;
  fiat_p384_mulx_u64(&x237, &x238, x3, (arg2[3]));
  uint64_t x239;
  uint64_t x240;
  fiat_p384_mulx_u64(&x239, &x240, x3, (arg2[2]));
  uint64_t x241;
  uint64_t x242;
  fiat_p384_mulx_u64(&x241, &x242, x3, (arg2[1]));
  uint64_t x243;
  uint64_t x244;
  fiat_p384_mulx_u64(&x243, &x244, x3, (arg2[0]));
  uint64_t x245;
  fiat_p384_uint1 x246;
  fiat_p384_addcarryx_u64(&x245, &x246, 0x0, x241, x244);
  uint64_t x247;
  fiat_p384_uint1 x248;
  fiat_p384_addcarryx_u64(&x247, &x248, x246, x239, x242);
  uint64_t x249;
  fiat_p384_uint1 x250;
  fiat_p384_addcarryx_u64(&x249, &x250, x248, x237, x240);
  uint64_t x251;
  fiat_p384_uint1 x252;
  fiat_p384_addcarryx_u64(&x251, &x252, x250, x235, x238);
  uint64_t x253;
  fiat_p384_uint1 x254;
  fiat_p384_addcarryx_u64(&x253, &x254, x252, x233, x236);
  uint64_t x255;
  fiat_p384_uint1 x256;
  fiat_p384_addcarryx_u64(&x255, &x256, x254, 0x0, x234);
  uint64_t x257;
  fiat_p384_uint1 x258;
  fiat_p384_addcarryx_u64(&x257, &x258, 0x0, x243, x219);
  uint64_t x259;
  fiat_p384_uint1 x260;
  fiat_p384_addcarryx_u64(&x259, &x260, x258, x245, x221);
  uint64_t x261;
  fiat_p384_uint1 x262;
  fiat_p384_addcarryx_u64(&x261, &x262, x260, x247, x223);
  uint64_t x263;
  fiat_p384_uint1 x264;
  fiat_p384_addcarryx_u64(&x263, &x264, x262, x249, x225);
  uint64_t x265;
  fiat_p384_uint1 x266;
  fiat_p384_addcarryx_u64(&x265, &x266, x264, x251, x227);
  uint64_t x267;
  fiat_p384_uint1 x268;
  fiat_p384_addcarryx_u64(&x267, &x268, x266, x253, x229);
  uint64_t x269;
  fiat_p384_uint1 x270;
  fiat_p384_addcarryx_u64(&x269, &x270, x268, x255, x231);
  uint64_t x271;
  uint64_t x272;
  fiat_p384_mulx_u64(&x271, &x272, x257, UINT64_C(0x100000001));
  uint64_t x273;
  uint64_t x274;
  fiat_p384_mulx_u64(&x273, &x274, x271, UINT64_C(0xffffffffffffffff));
  uint64_t x275;
  uint64_t x276;
  fiat_p384_mulx_u64(&x275, &x276, x271, UINT64_C(0xffffffffffffffff));
  uint64_t x277;
  uint64_t x278;
  fiat_p384_mulx_u64(&x277, &x278, x271, UINT64_C(0xffffffffffffffff));
  uint64_t x279;
  uint64_t x280;
  fiat_p384_mulx_u64(&x279, &x280, x271, UINT64_C(0xfffffffffffffffe));
  uint64_t x281;
  uint64_t x282;
  fiat_p384_mulx_u64(&x281, &x282, x271, UINT64_C(0xffffffff00000000));
  uint64_t x283;
  uint64_t x284;
  fiat_p384_mulx_u64(&x283, &x284, x271, UINT32_C(0xffffffff));
  uint64_t x285;
  fiat_p384_uint1 x286;
  fiat_p384_addcarryx_u64(&x285, &x286, 0x0, x281, x284);
  uint64_t x287;
  fiat_p384_uint1 x288;
  fiat_p384_addcarryx_u64(&x287, &x288, x286, x279, x282);
  uint64_t x289;
  fiat_p384_uint1 x290;
  fiat_p384_addcarryx_u64(&x289, &x290, x288, x277, x280);
  uint64_t x291;
  fiat_p384_uint1 x292;
  fiat_p384_addcarryx_u64(&x291, &x292, x290, x275, x278);
  uint64_t x293;
  fiat_p384_uint1 x294;
  fiat_p384_addcarryx_u64(&x293, &x294, x292, x273, x276);
  uint64_t x295;
  fiat_p384_uint1 x296;
  fiat_p384_addcarryx_u64(&x295, &x296, x294, 0x0, x274);
  uint64_t x297;
  fiat_p384_uint1 x298;
  fiat_p384_addcarryx_u64(&x297, &x298, 0x0, x283, x257);
  uint64_t x299;
  fiat_p384_uint1 x300;
  fiat_p384_addcarryx_u64(&x299, &x300, x298, x285, x259);
  uint64_t x301;
  fiat_p384_uint1 x302;
  fiat_p384_addcarryx_u64(&x301, &x302, x300, x287, x261);
  uint64_t x303;
  fiat_p384_uint1 x304;
  fiat_p384_addcarryx_u64(&x303, &x304, x302, x289, x263);
  uint64_t x305;
  fiat_p384_uint1 x306;
  fiat_p384_addcarryx_u64(&x305, &x306, x304, x291, x265);
  uint64_t x307;
  fiat_p384_uint1 x308;
  fiat_p384_addcarryx_u64(&x307, &x308, x306, x293, x267);
  uint64_t x309;
  fiat_p384_uint1 x310;
  fiat_p384_addcarryx_u64(&x309, &x310, x308, x295, x269);
  uint64_t x311;
  fiat_p384_uint1 x312;
  fiat_p384_addcarryx_u64(&x311, &x312, x310, 0x0, x270);
  uint64_t x313;
  uint64_t x314;
  fiat_p384_mulx_u64(&x313, &x314, x4, (arg2[5]));
  uint64_t x315;
  uint64_t x316;
  fiat_p384_mulx_u64(&x315, &x316, x4, (arg2[4]));
  uint64_t x317;
  uint64_t x318;
  fiat_p384_mulx_u64(&x317, &x318, x4, (arg2[3]));
  uint64_t x319;
  uint64_t x320;
  fiat_p384_mulx_u64(&x319, &x320, x4, (arg2[2]));
  uint64_t x321;
  uint64_t x322;
  fiat_p384_mulx_u64(&x321, &x322, x4, (arg2[1]));
  uint64_t x323;
  uint64_t x324;
  fiat_p384_mulx_u64(&x323, &x324, x4, (arg2[0]));
  uint64_t x325;
  fiat_p384_uint1 x326;
  fiat_p384_addcarryx_u64(&x325, &x326, 0x0, x321, x324);
  uint64_t x327;
  fiat_p384_uint1 x328;
  fiat_p384_addcarryx_u64(&x327, &x328, x326, x319, x322);
  uint64_t x329;
  fiat_p384_uint1 x330;
  fiat_p384_addcarryx_u64(&x329, &x330, x328, x317, x320);
  uint64_t x331;
  fiat_p384_uint1 x332;
  fiat_p384_addcarryx_u64(&x331, &x332, x330, x315, x318);
  uint64_t x333;
  fiat_p384_uint1 x334;
  fiat_p384_addcarryx_u64(&x333, &x334, x332, x313, x316);
  uint64_t x335;
  fiat_p384_uint1 x336;
  fiat_p384_addcarryx_u64(&x335, &x336, x334, 0x0, x314);
  uint64_t x337;
  fiat_p384_uint1 x338;
  fiat_p384_addcarryx_u64(&x337, &x338, 0x0, x323, x299);
  uint64_t x339;
  fiat_p384_uint1 x340;
  fiat_p384_addcarryx_u64(&x339, &x340, x338, x325, x301);
  uint64_t x341;
  fiat_p384_uint1 x342;
  fiat_p384_addcarryx_u64(&x341, &x342, x340, x327, x303);
  uint64_t x343;
  fiat_p384_uint1 x344;
  fiat_p384_addcarryx_u64(&x343, &x344, x342, x329, x305);
  uint64_t x345;
  fiat_p384_uint1 x346;
  fiat_p384_addcarryx_u64(&x345, &x346, x344, x331, x307);
  uint64_t x347;
  fiat_p384_uint1 x348;
  fiat_p384_addcarryx_u64(&x347, &x348, x346, x333, x309);
  uint64_t x349;
  fiat_p384_uint1 x350;
  fiat_p384_addcarryx_u64(&x349, &x350, x348, x335, x311);
  uint64_t x351;
  uint64_t x352;
  fiat_p384_mulx_u64(&x351, &x352, x337, UINT64_C(0x100000001));
  uint64_t x353;
  uint64_t x354;
  fiat_p384_mulx_u64(&x353, &x354, x351, UINT64_C(0xffffffffffffffff));
  uint64_t x355;
  uint64_t x356;
  fiat_p384_mulx_u64(&x355, &x356, x351, UINT64_C(0xffffffffffffffff));
  uint64_t x357;
  uint64_t x358;
  fiat_p384_mulx_u64(&x357, &x358, x351, UINT64_C(0xffffffffffffffff));
  uint64_t x359;
  uint64_t x360;
  fiat_p384_mulx_u64(&x359, &x360, x351, UINT64_C(0xfffffffffffffffe));
  uint64_t x361;
  uint64_t x362;
  fiat_p384_mulx_u64(&x361, &x362, x351, UINT64_C(0xffffffff00000000));
  uint64_t x363;
  uint64_t x364;
  fiat_p384_mulx_u64(&x363, &x364, x351, UINT32_C(0xffffffff));
  uint64_t x365;
  fiat_p384_uint1 x366;
  fiat_p384_addcarryx_u64(&x365, &x366, 0x0, x361, x364);
  uint64_t x367;
  fiat_p384_uint1 x368;
  fiat_p384_addcarryx_u64(&x367, &x368, x366, x359, x362);
  uint64_t x369;
  fiat_p384_uint1 x370;
  fiat_p384_addcarryx_u64(&x369, &x370, x368, x357, x360);
  uint64_t x371;
  fiat_p384_uint1 x372;
  fiat_p384_addcarryx_u64(&x371, &x372, x370, x355, x358);
  uint64_t x373;
  fiat_p384_uint1 x374;
  fiat_p384_addcarryx_u64(&x373, &x374, x372, x353, x356);
  uint64_t x375;
  fiat_p384_uint1 x376;
  fiat_p384_addcarryx_u64(&x375, &x376, x374, 0x0, x354);
  uint64_t x377;
  fiat_p384_uint1 x378;
  fiat_p384_addcarryx_u64(&x377, &x378, 0x0, x363, x337);
  uint64_t x379;
  fiat_p384_uint1 x380;
  fiat_p384_addcarryx_u64(&x379, &x380, x378, x365, x339);
  uint64_t x381;
  fiat_p384_uint1 x382;
  fiat_p384_addcarryx_u64(&x381, &x382, x380, x367, x341);
  uint64_t x383;
  fiat_p384_uint1 x384;
  fiat_p384_addcarryx_u64(&x383, &x384, x382, x369, x343);
  uint64_t x385;
  fiat_p384_uint1 x386;
  fiat_p384_addcarryx_u64(&x385, &x386, x384, x371, x345);
  uint64_t x387;
  fiat_p384_uint1 x388;
  fiat_p384_addcarryx_u64(&x387, &x388, x386, x373, x347);
  uint64_t x389;
  fiat_p384_uint1 x390;
  fiat_p384_addcarryx_u64(&x389, &x390, x388, x375, x349);
  uint64_t x391;
  fiat_p384_uint1 x392;
  fiat_p384_addcarryx_u64(&x391, &x392, x390, 0x0, x350);
  uint64_t x393;
  uint64_t x394;
  fiat_p384_mulx_u64(&x393, &x394, x5, (arg2[5]));
  uint64_t x395;
  uint64_t x396;
  fiat_p384_mulx_u64(&x395, &x396, x5, (arg2[4]));
  uint64_t x397;
  uint64_t x398;
  fiat_p384_mulx_u64(&x397, &x398, x5, (arg2[3]));
  uint64_t x399;
  uint64_t x400;
  fiat_p384_mulx_u64(&x399, &x400, x5, (arg2[2]));
  uint64_t x401;
  uint64_t x402;
  fiat_p384_mulx_u64(&x401, &x402, x5, (arg2[1]));
  uint64_t x403;
  uint64_t x404;
  fiat_p384_mulx_u64(&x403, &x404, x5, (arg2[0]));
  uint64_t x405;
  fiat_p384_uint1 x406;
  fiat_p384_addcarryx_u64(&x405, &x406, 0x0, x401, x404);
  uint64_t x407;
  fiat_p384_uint1 x408;
  fiat_p384_addcarryx_u64(&x407, &x408, x406, x399, x402);
  uint64_t x409;
  fiat_p384_uint1 x410;
  fiat_p384_addcarryx_u64(&x409, &x410, x408, x397, x400);
  uint64_t x411;
  fiat_p384_uint1 x412;
  fiat_p384_addcarryx_u64(&x411, &x412, x410, x395, x398);
  uint64_t x413;
  fiat_p384_uint1 x414;
  fiat_p384_addcarryx_u64(&x413, &x414, x412, x393, x396);
  uint64_t x415;
  fiat_p384_uint1 x416;
  fiat_p384_addcarryx_u64(&x415, &x416, x414, 0x0, x394);
  uint64_t x417;
  fiat_p384_uint1 x418;
  fiat_p384_addcarryx_u64(&x417, &x418, 0x0, x403, x379);
  uint64_t x419;
  fiat_p384_uint1 x420;
  fiat_p384_addcarryx_u64(&x419, &x420, x418, x405, x381);
  uint64_t x421;
  fiat_p384_uint1 x422;
  fiat_p384_addcarryx_u64(&x421, &x422, x420, x407, x383);
  uint64_t x423;
  fiat_p384_uint1 x424;
  fiat_p384_addcarryx_u64(&x423, &x424, x422, x409, x385);
  uint64_t x425;
  fiat_p384_uint1 x426;
  fiat_p384_addcarryx_u64(&x425, &x426, x424, x411, x387);
  uint64_t x427;
  fiat_p384_uint1 x428;
  fiat_p384_addcarryx_u64(&x427, &x428, x426, x413, x389);
  uint64_t x429;
  fiat_p384_uint1 x430;
  fiat_p384_addcarryx_u64(&x429, &x430, x428, x415, x391);
  uint64_t x431;
  uint64_t x432;
  fiat_p384_mulx_u64(&x431, &x432, x417, UINT64_C(0x100000001));
  uint64_t x433;
  uint64_t x434;
  fiat_p384_mulx_u64(&x433, &x434, x431, UINT64_C(0xffffffffffffffff));
  uint64_t x435;
  uint64_t x436;
  fiat_p384_mulx_u64(&x435, &x436, x431, UINT64_C(0xffffffffffffffff));
  uint64_t x437;
  uint64_t x438;
  fiat_p384_mulx_u64(&x437, &x438, x431, UINT64_C(0xffffffffffffffff));
  uint64_t x439;
  uint64_t x440;
  fiat_p384_mulx_u64(&x439, &x440, x431, UINT64_C(0xfffffffffffffffe));
  uint64_t x441;
  uint64_t x442;
  fiat_p384_mulx_u64(&x441, &x442, x431, UINT64_C(0xffffffff00000000));
  uint64_t x443;
  uint64_t x444;
  fiat_p384_mulx_u64(&x443, &x444, x431, UINT32_C(0xffffffff));
  uint64_t x445;
  fiat_p384_uint1 x446;
  fiat_p384_addcarryx_u64(&x445, &x446, 0x0, x441, x444);
  uint64_t x447;
  fiat_p384_uint1 x448;
  fiat_p384_addcarryx_u64(&x447, &x448, x446, x439, x442);
  uint64_t x449;
  fiat_p384_uint1 x450;
  fiat_p384_addcarryx_u64(&x449, &x450, x448, x437, x440);
  uint64_t x451;
  fiat_p384_uint1 x452;
  fiat_p384_addcarryx_u64(&x451, &x452, x450, x435, x438);
  uint64_t x453;
  fiat_p384_uint1 x454;
  fiat_p384_addcarryx_u64(&x453, &x454, x452, x433, x436);
  uint64_t x455;
  fiat_p384_uint1 x456;
  fiat_p384_addcarryx_u64(&x455, &x456, x454, 0x0, x434);
  uint64_t x457;
  fiat_p384_uint1 x458;
  fiat_p384_addcarryx_u64(&x457, &x458, 0x0, x443, x417);
  uint64_t x459;
  fiat_p384_uint1 x460;
  fiat_p384_addcarryx_u64(&x459, &x460, x458, x445, x419);
  uint64_t x461;
  fiat_p384_uint1 x462;
  fiat_p384_addcarryx_u64(&x461, &x462, x460, x447, x421);
  uint64_t x463;
  fiat_p384_uint1 x464;
  fiat_p384_addcarryx_u64(&x463, &x464, x462, x449, x423);
  uint64_t x465;
  fiat_p384_uint1 x466;
  fiat_p384_addcarryx_u64(&x465, &x466, x464, x451, x425);
  uint64_t x467;
  fiat_p384_uint1 x468;
  fiat_p384_addcarryx_u64(&x467, &x468, x466, x453, x427);
  uint64_t x469;
  fiat_p384_uint1 x470;
  fiat_p384_addcarryx_u64(&x469, &x470, x468, x455, x429);
  uint64_t x471;
  fiat_p384_uint1 x472;
  fiat_p384_addcarryx_u64(&x471, &x472, x470, 0x0, x430);
  uint64_t x473;
  fiat_p384_uint1 x474;
  fiat_p384_subborrowx_u64(&x473, &x474, 0x0, x459, UINT32_C(0xffffffff));
  uint64_t x475;
  fiat_p384_uint1 x476;
  fiat_p384_subborrowx_u64(&x475, &x476, x474, x461, UINT64_C(0xffffffff00000000));
  uint64_t x477;
  fiat_p384_uint1 x478;
  fiat_p384_subborrowx_u64(&x477, &x478, x476, x463, UINT64_C(0xfffffffffffffffe));
  uint64_t x479;
  fiat_p384_uint1 x480;
  fiat_p384_subborrowx_u64(&x479, &x480, x478, x465, UINT64_C(0xffffffffffffffff));
  uint64_t x481;
  fiat_p384_uint1 x482;
  fiat_p384_subborrowx_u64(&x481, &x482, x480, x467, UINT64_C(0xffffffffffffffff));
  uint64_t x483;
  fiat_p384_uint1 x484;
  fiat_p384_subborrowx_u64(&x483, &x484, x482, x469, UINT64_C(0xffffffffffffffff));
  uint64_t x485;
  fiat_p384_uint1 x486;
  fiat_p384_subborrowx_u64(&x485, &x486, x484, x471, 0x0);
  uint64_t x487;
  fiat_p384_cmovznz_u64(&x487, x486, x473, x459);
  uint64_t x488;
  fiat_p384_cmovznz_u64(&x488, x486, x475, x461);
  uint64_t x489;
  fiat_p384_cmovznz_u64(&x489, x486, x477, x463);
  uint64_t x490;
  fiat_p384_cmovznz_u64(&x490, x486, x479, x465);
  uint64_t x491;
  fiat_p384_cmovznz_u64(&x491, x486, x481, x467);
  uint64_t x492;
  fiat_p384_cmovznz_u64(&x492, x486, x483, x469);
  out1[0] = x487;
  out1[1] = x488;
  out1[2] = x489;
  out1[3] = x490;
  out1[4] = x491;
  out1[5] = x492;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_square(uint64_t out1[6], const uint64_t arg1[6]) {
  uint64_t x1 = (arg1[1]);
  uint64_t x2 = (arg1[2]);
  uint64_t x3 = (arg1[3]);
  uint64_t x4 = (arg1[4]);
  uint64_t x5 = (arg1[5]);
  uint64_t x6 = (arg1[0]);
  uint64_t x7;
  uint64_t x8;
  fiat_p384_mulx_u64(&x7, &x8, x6, (arg1[5]));
  uint64_t x9;
  uint64_t x10;
  fiat_p384_mulx_u64(&x9, &x10, x6, (arg1[4]));
  uint64_t x11;
  uint64_t x12;
  fiat_p384_mulx_u64(&x11, &x12, x6, (arg1[3]));
  uint64_t x13;
  uint64_t x14;
  fiat_p384_mulx_u64(&x13, &x14, x6, (arg1[2]));
  uint64_t x15;
  uint64_t x16;
  fiat_p384_mulx_u64(&x15, &x16, x6, (arg1[1]));
  uint64_t x17;
  uint64_t x18;
  fiat_p384_mulx_u64(&x17, &x18, x6, (arg1[0]));
  uint64_t x19;
  fiat_p384_uint1 x20;
  fiat_p384_addcarryx_u64(&x19, &x20, 0x0, x15, x18);
  uint64_t x21;
  fiat_p384_uint1 x22;
  fiat_p384_addcarryx_u64(&x21, &x22, x20, x13, x16);
  uint64_t x23;
  fiat_p384_uint1 x24;
  fiat_p384_addcarryx_u64(&x23, &x24, x22, x11, x14);
  uint64_t x25;
  fiat_p384_uint1 x26;
  fiat_p384_addcarryx_u64(&x25, &x26, x24, x9, x12);
  uint64_t x27;
  fiat_p384_uint1 x28;
  fiat_p384_addcarryx_u64(&x27, &x28, x26, x7, x10);
  uint64_t x29;
  fiat_p384_uint1 x30;
  fiat_p384_addcarryx_u64(&x29, &x30, x28, 0x0, x8);
  uint64_t x31;
  uint64_t x32;
  fiat_p384_mulx_u64(&x31, &x32, x17, UINT64_C(0x100000001));
  uint64_t x33;
  uint64_t x34;
  fiat_p384_mulx_u64(&x33, &x34, x31, UINT64_C(0xffffffffffffffff));
  uint64_t x35;
  uint64_t x36;
  fiat_p384_mulx_u64(&x35, &x36, x31, UINT64_C(0xffffffffffffffff));
  uint64_t x37;
  uint64_t x38;
  fiat_p384_mulx_u64(&x37, &x38, x31, UINT64_C(0xffffffffffffffff));
  uint64_t x39;
  uint64_t x40;
  fiat_p384_mulx_u64(&x39, &x40, x31, UINT64_C(0xfffffffffffffffe));
  uint64_t x41;
  uint64_t x42;
  fiat_p384_mulx_u64(&x41, &x42, x31, UINT64_C(0xffffffff00000000));
  uint64_t x43;
  uint64_t x44;
  fiat_p384_mulx_u64(&x43, &x44, x31, UINT32_C(0xffffffff));
  uint64_t x45;
  fiat_p384_uint1 x46;
  fiat_p384_addcarryx_u64(&x45, &x46, 0x0, x41, x44);
  uint64_t x47;
  fiat_p384_uint1 x48;
  fiat_p384_addcarryx_u64(&x47, &x48, x46, x39, x42);
  uint64_t x49;
  fiat_p384_uint1 x50;
  fiat_p384_addcarryx_u64(&x49, &x50, x48, x37, x40);
  uint64_t x51;
  fiat_p384_uint1 x52;
  fiat_p384_addcarryx_u64(&x51, &x52, x50, x35, x38);
  uint64_t x53;
  fiat_p384_uint1 x54;
  fiat_p384_addcarryx_u64(&x53, &x54, x52, x33, x36);
  uint64_t x55;
  fiat_p384_uint1 x56;
  fiat_p384_addcarryx_u64(&x55, &x56, x54, 0x0, x34);
  uint64_t x57;
  fiat_p384_uint1 x58;
  fiat_p384_addcarryx_u64(&x57, &x58, 0x0, x43, x17);
  uint64_t x59;
  fiat_p384_uint1 x60;
  fiat_p384_addcarryx_u64(&x59, &x60, x58, x45, x19);
  uint64_t x61;
  fiat_p384_uint1 x62;
  fiat_p384_addcarryx_u64(&x61, &x62, x60, x47, x21);
  uint64_t x63;
  fiat_p384_uint1 x64;
  fiat_p384_addcarryx_u64(&x63, &x64, x62, x49, x23);
  uint64_t x65;
  fiat_p384_uint1 x66;
  fiat_p384_addcarryx_u64(&x65, &x66, x64, x51, x25);
  uint64_t x67;
  fiat_p384_uint1 x68;
  fiat_p384_addcarryx_u64(&x67, &x68, x66, x53, x27);
  uint64_t x69;
  fiat_p384_uint1 x70;
  fiat_p384_addcarryx_u64(&x69, &x70, x68, x55, x29);
  uint64_t x71;
  fiat_p384_uint1 x72;
  fiat_p384_addcarryx_u64(&x71, &x72, x70, 0x0, 0x0);
  uint64_t x73;
  uint64_t x74;
  fiat_p384_mulx_u64(&x73, &x74, x1, (arg1[5]));
  uint64_t x75;
  uint64_t x76;
  fiat_p384_mulx_u64(&x75, &x76, x1, (arg1[4]));
  uint64_t x77;
  uint64_t x78;
  fiat_p384_mulx_u64(&x77, &x78, x1, (arg1[3]));
  uint64_t x79;
  uint64_t x80;
  fiat_p384_mulx_u64(&x79, &x80, x1, (arg1[2]));
  uint64_t x81;
  uint64_t x82;
  fiat_p384_mulx_u64(&x81, &x82, x1, (arg1[1]));
  uint64_t x83;
  uint64_t x84;
  fiat_p384_mulx_u64(&x83, &x84, x1, (arg1[0]));
  uint64_t x85;
  fiat_p384_uint1 x86;
  fiat_p384_addcarryx_u64(&x85, &x86, 0x0, x81, x84);
  uint64_t x87;
  fiat_p384_uint1 x88;
  fiat_p384_addcarryx_u64(&x87, &x88, x86, x79, x82);
  uint64_t x89;
  fiat_p384_uint1 x90;
  fiat_p384_addcarryx_u64(&x89, &x90, x88, x77, x80);
  uint64_t x91;
  fiat_p384_uint1 x92;
  fiat_p384_addcarryx_u64(&x91, &x92, x90, x75, x78);
  uint64_t x93;
  fiat_p384_uint1 x94;
  fiat_p384_addcarryx_u64(&x93, &x94, x92, x73, x76);
  uint64_t x95;
  fiat_p384_uint1 x96;
  fiat_p384_addcarryx_u64(&x95, &x96, x94, 0x0, x74);
  uint64_t x97;
  fiat_p384_uint1 x98;
  fiat_p384_addcarryx_u64(&x97, &x98, 0x0, x83, x59);
  uint64_t x99;
  fiat_p384_uint1 x100;
  fiat_p384_addcarryx_u64(&x99, &x100, x98, x85, x61);
  uint64_t x101;
  fiat_p384_uint1 x102;
  fiat_p384_addcarryx_u64(&x101, &x102, x100, x87, x63);
  uint64_t x103;
  fiat_p384_uint1 x104;
  fiat_p384_addcarryx_u64(&x103, &x104, x102, x89, x65);
  uint64_t x105;
  fiat_p384_uint1 x106;
  fiat_p384_addcarryx_u64(&x105, &x106, x104, x91, x67);
  uint64_t x107;
  fiat_p384_uint1 x108;
  fiat_p384_addcarryx_u64(&x107, &x108, x106, x93, x69);
  uint64_t x109;
  fiat_p384_uint1 x110;
  fiat_p384_addcarryx_u64(&x109, &x110, x108, x95, (fiat_p384_uint1)x71);
  uint64_t x111;
  uint64_t x112;
  fiat_p384_mulx_u64(&x111, &x112, x97, UINT64_C(0x100000001));
  uint64_t x113;
  uint64_t x114;
  fiat_p384_mulx_u64(&x113, &x114, x111, UINT64_C(0xffffffffffffffff));
  uint64_t x115;
  uint64_t x116;
  fiat_p384_mulx_u64(&x115, &x116, x111, UINT64_C(0xffffffffffffffff));
  uint64_t x117;
  uint64_t x118;
  fiat_p384_mulx_u64(&x117, &x118, x111, UINT64_C(0xffffffffffffffff));
  uint64_t x119;
  uint64_t x120;
  fiat_p384_mulx_u64(&x119, &x120, x111, UINT64_C(0xfffffffffffffffe));
  uint64_t x121;
  uint64_t x122;
  fiat_p384_mulx_u64(&x121, &x122, x111, UINT64_C(0xffffffff00000000));
  uint64_t x123;
  uint64_t x124;
  fiat_p384_mulx_u64(&x123, &x124, x111, UINT32_C(0xffffffff));
  uint64_t x125;
  fiat_p384_uint1 x126;
  fiat_p384_addcarryx_u64(&x125, &x126, 0x0, x121, x124);
  uint64_t x127;
  fiat_p384_uint1 x128;
  fiat_p384_addcarryx_u64(&x127, &x128, x126, x119, x122);
  uint64_t x129;
  fiat_p384_uint1 x130;
  fiat_p384_addcarryx_u64(&x129, &x130, x128, x117, x120);
  uint64_t x131;
  fiat_p384_uint1 x132;
  fiat_p384_addcarryx_u64(&x131, &x132, x130, x115, x118);
  uint64_t x133;
  fiat_p384_uint1 x134;
  fiat_p384_addcarryx_u64(&x133, &x134, x132, x113, x116);
  uint64_t x135;
  fiat_p384_uint1 x136;
  fiat_p384_addcarryx_u64(&x135, &x136, x134, 0x0, x114);
  uint64_t x137;
  fiat_p384_uint1 x138;
  fiat_p384_addcarryx_u64(&x137, &x138, 0x0, x123, x97);
  uint64_t x139;
  fiat_p384_uint1 x140;
  fiat_p384_addcarryx_u64(&x139, &x140, x138, x125, x99);
  uint64_t x141;
  fiat_p384_uint1 x142;
  fiat_p384_addcarryx_u64(&x141, &x142, x140, x127, x101);
  uint64_t x143;
  fiat_p384_uint1 x144;
  fiat_p384_addcarryx_u64(&x143, &x144, x142, x129, x103);
  uint64_t x145;
  fiat_p384_uint1 x146;
  fiat_p384_addcarryx_u64(&x145, &x146, x144, x131, x105);
  uint64_t x147;
  fiat_p384_uint1 x148;
  fiat_p384_addcarryx_u64(&x147, &x148, x146, x133, x107);
  uint64_t x149;
  fiat_p384_uint1 x150;
  fiat_p384_addcarryx_u64(&x149, &x150, x148, x135, x109);
  uint64_t x151;
  fiat_p384_uint1 x152;
  fiat_p384_addcarryx_u64(&x151, &x152, x150, 0x0, x110);
  uint64_t x153;
  uint64_t x154;
  fiat_p384_mulx_u64(&x153, &x154, x2, (arg1[5]));
  uint64_t x155;
  uint64_t x156;
  fiat_p384_mulx_u64(&x155, &x156, x2, (arg1[4]));
  uint64_t x157;
  uint64_t x158;
  fiat_p384_mulx_u64(&x157, &x158, x2, (arg1[3]));
  uint64_t x159;
  uint64_t x160;
  fiat_p384_mulx_u64(&x159, &x160, x2, (arg1[2]));
  uint64_t x161;
  uint64_t x162;
  fiat_p384_mulx_u64(&x161, &x162, x2, (arg1[1]));
  uint64_t x163;
  uint64_t x164;
  fiat_p384_mulx_u64(&x163, &x164, x2, (arg1[0]));
  uint64_t x165;
  fiat_p384_uint1 x166;
  fiat_p384_addcarryx_u64(&x165, &x166, 0x0, x161, x164);
  uint64_t x167;
  fiat_p384_uint1 x168;
  fiat_p384_addcarryx_u64(&x167, &x168, x166, x159, x162);
  uint64_t x169;
  fiat_p384_uint1 x170;
  fiat_p384_addcarryx_u64(&x169, &x170, x168, x157, x160);
  uint64_t x171;
  fiat_p384_uint1 x172;
  fiat_p384_addcarryx_u64(&x171, &x172, x170, x155, x158);
  uint64_t x173;
  fiat_p384_uint1 x174;
  fiat_p384_addcarryx_u64(&x173, &x174, x172, x153, x156);
  uint64_t x175;
  fiat_p384_uint1 x176;
  fiat_p384_addcarryx_u64(&x175, &x176, x174, 0x0, x154);
  uint64_t x177;
  fiat_p384_uint1 x178;
  fiat_p384_addcarryx_u64(&x177, &x178, 0x0, x163, x139);
  uint64_t x179;
  fiat_p384_uint1 x180;
  fiat_p384_addcarryx_u64(&x179, &x180, x178, x165, x141);
  uint64_t x181;
  fiat_p384_uint1 x182;
  fiat_p384_addcarryx_u64(&x181, &x182, x180, x167, x143);
  uint64_t x183;
  fiat_p384_uint1 x184;
  fiat_p384_addcarryx_u64(&x183, &x184, x182, x169, x145);
  uint64_t x185;
  fiat_p384_uint1 x186;
  fiat_p384_addcarryx_u64(&x185, &x186, x184, x171, x147);
  uint64_t x187;
  fiat_p384_uint1 x188;
  fiat_p384_addcarryx_u64(&x187, &x188, x186, x173, x149);
  uint64_t x189;
  fiat_p384_uint1 x190;
  fiat_p384_addcarryx_u64(&x189, &x190, x188, x175, x151);
  uint64_t x191;
  uint64_t x192;
  fiat_p384_mulx_u64(&x191, &x192, x177, UINT64_C(0x100000001));
  uint64_t x193;
  uint64_t x194;
  fiat_p384_mulx_u64(&x193, &x194, x191, UINT64_C(0xffffffffffffffff));
  uint64_t x195;
  uint64_t x196;
  fiat_p384_mulx_u64(&x195, &x196, x191, UINT64_C(0xffffffffffffffff));
  uint64_t x197;
  uint64_t x198;
  fiat_p384_mulx_u64(&x197, &x198, x191, UINT64_C(0xffffffffffffffff));
  uint64_t x199;
  uint64_t x200;
  fiat_p384_mulx_u64(&x199, &x200, x191, UINT64_C(0xfffffffffffffffe));
  uint64_t x201;
  uint64_t x202;
  fiat_p384_mulx_u64(&x201, &x202, x191, UINT64_C(0xffffffff00000000));
  uint64_t x203;
  uint64_t x204;
  fiat_p384_mulx_u64(&x203, &x204, x191, UINT32_C(0xffffffff));
  uint64_t x205;
  fiat_p384_uint1 x206;
  fiat_p384_addcarryx_u64(&x205, &x206, 0x0, x201, x204);
  uint64_t x207;
  fiat_p384_uint1 x208;
  fiat_p384_addcarryx_u64(&x207, &x208, x206, x199, x202);
  uint64_t x209;
  fiat_p384_uint1 x210;
  fiat_p384_addcarryx_u64(&x209, &x210, x208, x197, x200);
  uint64_t x211;
  fiat_p384_uint1 x212;
  fiat_p384_addcarryx_u64(&x211, &x212, x210, x195, x198);
  uint64_t x213;
  fiat_p384_uint1 x214;
  fiat_p384_addcarryx_u64(&x213, &x214, x212, x193, x196);
  uint64_t x215;
  fiat_p384_uint1 x216;
  fiat_p384_addcarryx_u64(&x215, &x216, x214, 0x0, x194);
  uint64_t x217;
  fiat_p384_uint1 x218;
  fiat_p384_addcarryx_u64(&x217, &x218, 0x0, x203, x177);
  uint64_t x219;
  fiat_p384_uint1 x220;
  fiat_p384_addcarryx_u64(&x219, &x220, x218, x205, x179);
  uint64_t x221;
  fiat_p384_uint1 x222;
  fiat_p384_addcarryx_u64(&x221, &x222, x220, x207, x181);
  uint64_t x223;
  fiat_p384_uint1 x224;
  fiat_p384_addcarryx_u64(&x223, &x224, x222, x209, x183);
  uint64_t x225;
  fiat_p384_uint1 x226;
  fiat_p384_addcarryx_u64(&x225, &x226, x224, x211, x185);
  uint64_t x227;
  fiat_p384_uint1 x228;
  fiat_p384_addcarryx_u64(&x227, &x228, x226, x213, x187);
  uint64_t x229;
  fiat_p384_uint1 x230;
  fiat_p384_addcarryx_u64(&x229, &x230, x228, x215, x189);
  uint64_t x231;
  fiat_p384_uint1 x232;
  fiat_p384_addcarryx_u64(&x231, &x232, x230, 0x0, x190);
  uint64_t x233;
  uint64_t x234;
  fiat_p384_mulx_u64(&x233, &x234, x3, (arg1[5]));
  uint64_t x235;
  uint64_t x236;
  fiat_p384_mulx_u64(&x235, &x236, x3, (arg1[4]));
  uint64_t x237;
  uint64_t x238;
  fiat_p384_mulx_u64(&x237, &x238, x3, (arg1[3]));
  uint64_t x239;
  uint64_t x240;
  fiat_p384_mulx_u64(&x239, &x240, x3, (arg1[2]));
  uint64_t x241;
  uint64_t x242;
  fiat_p384_mulx_u64(&x241, &x242, x3, (arg1[1]));
  uint64_t x243;
  uint64_t x244;
  fiat_p384_mulx_u64(&x243, &x244, x3, (arg1[0]));
  uint64_t x245;
  fiat_p384_uint1 x246;
  fiat_p384_addcarryx_u64(&x245, &x246, 0x0, x241, x244);
  uint64_t x247;
  fiat_p384_uint1 x248;
  fiat_p384_addcarryx_u64(&x247, &x248, x246, x239, x242);
  uint64_t x249;
  fiat_p384_uint1 x250;
  fiat_p384_addcarryx_u64(&x249, &x250, x248, x237, x240);
  uint64_t x251;
  fiat_p384_uint1 x252;
  fiat_p384_addcarryx_u64(&x251, &x252, x250, x235, x238);
  uint64_t x253;
  fiat_p384_uint1 x254;
  fiat_p384_addcarryx_u64(&x253, &x254, x252, x233, x236);
  uint64_t x255;
  fiat_p384_uint1 x256;
  fiat_p384_addcarryx_u64(&x255, &x256, x254, 0x0, x234);
  uint64_t x257;
  fiat_p384_uint1 x258;
  fiat_p384_addcarryx_u64(&x257, &x258, 0x0, x243, x219);
  uint64_t x259;
  fiat_p384_uint1 x260;
  fiat_p384_addcarryx_u64(&x259, &x260, x258, x245, x221);
  uint64_t x261;
  fiat_p384_uint1 x262;
  fiat_p384_addcarryx_u64(&x261, &x262, x260, x247, x223);
  uint64_t x263;
  fiat_p384_uint1 x264;
  fiat_p384_addcarryx_u64(&x263, &x264, x262, x249, x225);
  uint64_t x265;
  fiat_p384_uint1 x266;
  fiat_p384_addcarryx_u64(&x265, &x266, x264, x251, x227);
  uint64_t x267;
  fiat_p384_uint1 x268;
  fiat_p384_addcarryx_u64(&x267, &x268, x266, x253, x229);
  uint64_t x269;
  fiat_p384_uint1 x270;
  fiat_p384_addcarryx_u64(&x269, &x270, x268, x255, x231);
  uint64_t x271;
  uint64_t x272;
  fiat_p384_mulx_u64(&x271, &x272, x257, UINT64_C(0x100000001));
  uint64_t x273;
  uint64_t x274;
  fiat_p384_mulx_u64(&x273, &x274, x271, UINT64_C(0xffffffffffffffff));
  uint64_t x275;
  uint64_t x276;
  fiat_p384_mulx_u64(&x275, &x276, x271, UINT64_C(0xffffffffffffffff));
  uint64_t x277;
  uint64_t x278;
  fiat_p384_mulx_u64(&x277, &x278, x271, UINT64_C(0xffffffffffffffff));
  uint64_t x279;
  uint64_t x280;
  fiat_p384_mulx_u64(&x279, &x280, x271, UINT64_C(0xfffffffffffffffe));
  uint64_t x281;
  uint64_t x282;
  fiat_p384_mulx_u64(&x281, &x282, x271, UINT64_C(0xffffffff00000000));
  uint64_t x283;
  uint64_t x284;
  fiat_p384_mulx_u64(&x283, &x284, x271, UINT32_C(0xffffffff));
  uint64_t x285;
  fiat_p384_uint1 x286;
  fiat_p384_addcarryx_u64(&x285, &x286, 0x0, x281, x284);
  uint64_t x287;
  fiat_p384_uint1 x288;
  fiat_p384_addcarryx_u64(&x287, &x288, x286, x279, x282);
  uint64_t x289;
  fiat_p384_uint1 x290;
  fiat_p384_addcarryx_u64(&x289, &x290, x288, x277, x280);
  uint64_t x291;
  fiat_p384_uint1 x292;
  fiat_p384_addcarryx_u64(&x291, &x292, x290, x275, x278);
  uint64_t x293;
  fiat_p384_uint1 x294;
  fiat_p384_addcarryx_u64(&x293, &x294, x292, x273, x276);
  uint64_t x295;
  fiat_p384_uint1 x296;
  fiat_p384_addcarryx_u64(&x295, &x296, x294, 0x0, x274);
  uint64_t x297;
  fiat_p384_uint1 x298;
  fiat_p384_addcarryx_u64(&x297, &x298, 0x0, x283, x257);
  uint64_t x299;
  fiat_p384_uint1 x300;
  fiat_p384_addcarryx_u64(&x299, &x300, x298, x285, x259);
  uint64_t x301;
  fiat_p384_uint1 x302;
  fiat_p384_addcarryx_u64(&x301, &x302, x300, x287, x261);
  uint64_t x303;
  fiat_p384_uint1 x304;
  fiat_p384_addcarryx_u64(&x303, &x304, x302, x289, x263);
  uint64_t x305;
  fiat_p384_uint1 x306;
  fiat_p384_addcarryx_u64(&x305, &x306, x304, x291, x265);
  uint64_t x307;
  fiat_p384_uint1 x308;
  fiat_p384_addcarryx_u64(&x307, &x308, x306, x293, x267);
  uint64_t x309;
  fiat_p384_uint1 x310;
  fiat_p384_addcarryx_u64(&x309, &x310, x308, x295, x269);
  uint64_t x311;
  fiat_p384_uint1 x312;
  fiat_p384_addcarryx_u64(&x311, &x312, x310, 0x0, x270);
  uint64_t x313;
  uint64_t x314;
  fiat_p384_mulx_u64(&x313, &x314, x4, (arg1[5]));
  uint64_t x315;
  uint64_t x316;
  fiat_p384_mulx_u64(&x315, &x316, x4, (arg1[4]));
  uint64_t x317;
  uint64_t x318;
  fiat_p384_mulx_u64(&x317, &x318, x4, (arg1[3]));
  uint64_t x319;
  uint64_t x320;
  fiat_p384_mulx_u64(&x319, &x320, x4, (arg1[2]));
  uint64_t x321;
  uint64_t x322;
  fiat_p384_mulx_u64(&x321, &x322, x4, (arg1[1]));
  uint64_t x323;
  uint64_t x324;
  fiat_p384_mulx_u64(&x323, &x324, x4, (arg1[0]));
  uint64_t x325;
  fiat_p384_uint1 x326;
  fiat_p384_addcarryx_u64(&x325, &x326, 0x0, x321, x324);
  uint64_t x327;
  fiat_p384_uint1 x328;
  fiat_p384_addcarryx_u64(&x327, &x328, x326, x319, x322);
  uint64_t x329;
  fiat_p384_uint1 x330;
  fiat_p384_addcarryx_u64(&x329, &x330, x328, x317, x320);
  uint64_t x331;
  fiat_p384_uint1 x332;
  fiat_p384_addcarryx_u64(&x331, &x332, x330, x315, x318);
  uint64_t x333;
  fiat_p384_uint1 x334;
  fiat_p384_addcarryx_u64(&x333, &x334, x332, x313, x316);
  uint64_t x335;
  fiat_p384_uint1 x336;
  fiat_p384_addcarryx_u64(&x335, &x336, x334, 0x0, x314);
  uint64_t x337;
  fiat_p384_uint1 x338;
  fiat_p384_addcarryx_u64(&x337, &x338, 0x0, x323, x299);
  uint64_t x339;
  fiat_p384_uint1 x340;
  fiat_p384_addcarryx_u64(&x339, &x340, x338, x325, x301);
  uint64_t x341;
  fiat_p384_uint1 x342;
  fiat_p384_addcarryx_u64(&x341, &x342, x340, x327, x303);
  uint64_t x343;
  fiat_p384_uint1 x344;
  fiat_p384_addcarryx_u64(&x343, &x344, x342, x329, x305);
  uint64_t x345;
  fiat_p384_uint1 x346;
  fiat_p384_addcarryx_u64(&x345, &x346, x344, x331, x307);
  uint64_t x347;
  fiat_p384_uint1 x348;
  fiat_p384_addcarryx_u64(&x347, &x348, x346, x333, x309);
  uint64_t x349;
  fiat_p384_uint1 x350;
  fiat_p384_addcarryx_u64(&x349, &x350, x348, x335, x311);
  uint64_t x351;
  uint64_t x352;
  fiat_p384_mulx_u64(&x351, &x352, x337, UINT64_C(0x100000001));
  uint64_t x353;
  uint64_t x354;
  fiat_p384_mulx_u64(&x353, &x354, x351, UINT64_C(0xffffffffffffffff));
  uint64_t x355;
  uint64_t x356;
  fiat_p384_mulx_u64(&x355, &x356, x351, UINT64_C(0xffffffffffffffff));
  uint64_t x357;
  uint64_t x358;
  fiat_p384_mulx_u64(&x357, &x358, x351, UINT64_C(0xffffffffffffffff));
  uint64_t x359;
  uint64_t x360;
  fiat_p384_mulx_u64(&x359, &x360, x351, UINT64_C(0xfffffffffffffffe));
  uint64_t x361;
  uint64_t x362;
  fiat_p384_mulx_u64(&x361, &x362, x351, UINT64_C(0xffffffff00000000));
  uint64_t x363;
  uint64_t x364;
  fiat_p384_mulx_u64(&x363, &x364, x351, UINT32_C(0xffffffff));
  uint64_t x365;
  fiat_p384_uint1 x366;
  fiat_p384_addcarryx_u64(&x365, &x366, 0x0, x361, x364);
  uint64_t x367;
  fiat_p384_uint1 x368;
  fiat_p384_addcarryx_u64(&x367, &x368, x366, x359, x362);
  uint64_t x369;
  fiat_p384_uint1 x370;
  fiat_p384_addcarryx_u64(&x369, &x370, x368, x357, x360);
  uint64_t x371;
  fiat_p384_uint1 x372;
  fiat_p384_addcarryx_u64(&x371, &x372, x370, x355, x358);
  uint64_t x373;
  fiat_p384_uint1 x374;
  fiat_p384_addcarryx_u64(&x373, &x374, x372, x353, x356);
  uint64_t x375;
  fiat_p384_uint1 x376;
  fiat_p384_addcarryx_u64(&x375, &x376, x374, 0x0, x354);
  uint64_t x377;
  fiat_p384_uint1 x378;
  fiat_p384_addcarryx_u64(&x377, &x378, 0x0, x363, x337);
  uint64_t x379;
  fiat_p384_uint1 x380;
  fiat_p384_addcarryx_u64(&x379, &x380, x378, x365, x339);
  uint64_t x381;
  fiat_p384_uint1 x382;
  fiat_p384_addcarryx_u64(&x381, &x382, x380, x367, x341);
  uint64_t x383;
  fiat_p384_uint1 x384;
  fiat_p384_addcarryx_u64(&x383, &x384, x382, x369, x343);
  uint64_t x385;
  fiat_p384_uint1 x386;
  fiat_p384_addcarryx_u64(&x385, &x386, x384, x371, x345);
  uint64_t x387;
  fiat_p384_uint1 x388;
  fiat_p384_addcarryx_u64(&x387, &x388, x386, x373, x347);
  uint64_t x389;
  fiat_p384_uint1 x390;
  fiat_p384_addcarryx_u64(&x389, &x390, x388, x375, x349);
  uint64_t x391;
  fiat_p384_uint1 x392;
  fiat_p384_addcarryx_u64(&x391, &x392, x390, 0x0, x350);
  uint64_t x393;
  uint64_t x394;
  fiat_p384_mulx_u64(&x393, &x394, x5, (arg1[5]));
  uint64_t x395;
  uint64_t x396;
  fiat_p384_mulx_u64(&x395, &x396, x5, (arg1[4]));
  uint64_t x397;
  uint64_t x398;
  fiat_p384_mulx_u64(&x397, &x398, x5, (arg1[3]));
  uint64_t x399;
  uint64_t x400;
  fiat_p384_mulx_u64(&x399, &x400, x5, (arg1[2]));
  uint64_t x401;
  uint64_t x402;
  fiat_p384_mulx_u64(&x401, &x402, x5, (arg1[1]));
  uint64_t x403;
  uint64_t x404;
  fiat_p384_mulx_u64(&x403, &x404, x5, (arg1[0]));
  uint64_t x405;
  fiat_p384_uint1 x406;
  fiat_p384_addcarryx_u64(&x405, &x406, 0x0, x401, x404);
  uint64_t x407;
  fiat_p384_uint1 x408;
  fiat_p384_addcarryx_u64(&x407, &x408, x406, x399, x402);
  uint64_t x409;
  fiat_p384_uint1 x410;
  fiat_p384_addcarryx_u64(&x409, &x410, x408, x397, x400);
  uint64_t x411;
  fiat_p384_uint1 x412;
  fiat_p384_addcarryx_u64(&x411, &x412, x410, x395, x398);
  uint64_t x413;
  fiat_p384_uint1 x414;
  fiat_p384_addcarryx_u64(&x413, &x414, x412, x393, x396);
  uint64_t x415;
  fiat_p384_uint1 x416;
  fiat_p384_addcarryx_u64(&x415, &x416, x414, 0x0, x394);
  uint64_t x417;
  fiat_p384_uint1 x418;
  fiat_p384_addcarryx_u64(&x417, &x418, 0x0, x403, x379);
  uint64_t x419;
  fiat_p384_uint1 x420;
  fiat_p384_addcarryx_u64(&x419, &x420, x418, x405, x381);
  uint64_t x421;
  fiat_p384_uint1 x422;
  fiat_p384_addcarryx_u64(&x421, &x422, x420, x407, x383);
  uint64_t x423;
  fiat_p384_uint1 x424;
  fiat_p384_addcarryx_u64(&x423, &x424, x422, x409, x385);
  uint64_t x425;
  fiat_p384_uint1 x426;
  fiat_p384_addcarryx_u64(&x425, &x426, x424, x411, x387);
  uint64_t x427;
  fiat_p384_uint1 x428;
  fiat_p384_addcarryx_u64(&x427, &x428, x426, x413, x389);
  uint64_t x429;
  fiat_p384_uint1 x430;
  fiat_p384_addcarryx_u64(&x429, &x430, x428, x415, x391);
  uint64_t x431;
  uint64_t x432;
  fiat_p384_mulx_u64(&x431, &x432, x417, UINT64_C(0x100000001));
  uint64_t x433;
  uint64_t x434;
  fiat_p384_mulx_u64(&x433, &x434, x431, UINT64_C(0xffffffffffffffff));
  uint64_t x435;
  uint64_t x436;
  fiat_p384_mulx_u64(&x435, &x436, x431, UINT64_C(0xffffffffffffffff));
  uint64_t x437;
  uint64_t x438;
  fiat_p384_mulx_u64(&x437, &x438, x431, UINT64_C(0xffffffffffffffff));
  uint64_t x439;
  uint64_t x440;
  fiat_p384_mulx_u64(&x439, &x440, x431, UINT64_C(0xfffffffffffffffe));
  uint64_t x441;
  uint64_t x442;
  fiat_p384_mulx_u64(&x441, &x442, x431, UINT64_C(0xffffffff00000000));
  uint64_t x443;
  uint64_t x444;
  fiat_p384_mulx_u64(&x443, &x444, x431, UINT32_C(0xffffffff));
  uint64_t x445;
  fiat_p384_uint1 x446;
  fiat_p384_addcarryx_u64(&x445, &x446, 0x0, x441, x444);
  uint64_t x447;
  fiat_p384_uint1 x448;
  fiat_p384_addcarryx_u64(&x447, &x448, x446, x439, x442);
  uint64_t x449;
  fiat_p384_uint1 x450;
  fiat_p384_addcarryx_u64(&x449, &x450, x448, x437, x440);
  uint64_t x451;
  fiat_p384_uint1 x452;
  fiat_p384_addcarryx_u64(&x451, &x452, x450, x435, x438);
  uint64_t x453;
  fiat_p384_uint1 x454;
  fiat_p384_addcarryx_u64(&x453, &x454, x452, x433, x436);
  uint64_t x455;
  fiat_p384_uint1 x456;
  fiat_p384_addcarryx_u64(&x455, &x456, x454, 0x0, x434);
  uint64_t x457;
  fiat_p384_uint1 x458;
  fiat_p384_addcarryx_u64(&x457, &x458, 0x0, x443, x417);
  uint64_t x459;
  fiat_p384_uint1 x460;
  fiat_p384_addcarryx_u64(&x459, &x460, x458, x445, x419);
  uint64_t x461;
  fiat_p384_uint1 x462;
  fiat_p384_addcarryx_u64(&x461, &x462, x460, x447, x421);
  uint64_t x463;
  fiat_p384_uint1 x464;
  fiat_p384_addcarryx_u64(&x463, &x464, x462, x449, x423);
  uint64_t x465;
  fiat_p384_uint1 x466;
  fiat_p384_addcarryx_u64(&x465, &x466, x464, x451, x425);
  uint64_t x467;
  fiat_p384_uint1 x468;
  fiat_p384_addcarryx_u64(&x467, &x468, x466, x453, x427);
  uint64_t x469;
  fiat_p384_uint1 x470;
  fiat_p384_addcarryx_u64(&x469, &x470, x468, x455, x429);
  uint64_t x471;
  fiat_p384_uint1 x472;
  fiat_p384_addcarryx_u64(&x471, &x472, x470, 0x0, x430);
  uint64_t x473;
  fiat_p384_uint1 x474;
  fiat_p384_subborrowx_u64(&x473, &x474, 0x0, x459, UINT32_C(0xffffffff));
  uint64_t x475;
  fiat_p384_uint1 x476;
  fiat_p384_subborrowx_u64(&x475, &x476, x474, x461, UINT64_C(0xffffffff00000000));
  uint64_t x477;
  fiat_p384_uint1 x478;
  fiat_p384_subborrowx_u64(&x477, &x478, x476, x463, UINT64_C(0xfffffffffffffffe));
  uint64_t x479;
  fiat_p384_uint1 x480;
  fiat_p384_subborrowx_u64(&x479, &x480, x478, x465, UINT64_C(0xffffffffffffffff));
  uint64_t x481;
  fiat_p384_uint1 x482;
  fiat_p384_subborrowx_u64(&x481, &x482, x480, x467, UINT64_C(0xffffffffffffffff));
  uint64_t x483;
  fiat_p384_uint1 x484;
  fiat_p384_subborrowx_u64(&x483, &x484, x482, x469, UINT64_C(0xffffffffffffffff));
  uint64_t x485;
  fiat_p384_uint1 x486;
  fiat_p384_subborrowx_u64(&x485, &x486, x484, x471, 0x0);
  uint64_t x487;
  fiat_p384_cmovznz_u64(&x487, x486, x473, x459);
  uint64_t x488;
  fiat_p384_cmovznz_u64(&x488, x486, x475, x461);
  uint64_t x489;
  fiat_p384_cmovznz_u64(&x489, x486, x477, x463);
  uint64_t x490;
  fiat_p384_cmovznz_u64(&x490, x486, x479, x465);
  uint64_t x491;
  fiat_p384_cmovznz_u64(&x491, x486, x481, x467);
  uint64_t x492;
  fiat_p384_cmovznz_u64(&x492, x486, x483, x469);
  out1[0] = x487;
  out1[1] = x488;
  out1[2] = x489;
  out1[3] = x490;
  out1[4] = x491;
  out1[5] = x492;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 *   arg2: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_add(uint64_t out1[6], const uint64_t arg1[6], const uint64_t arg2[6]) {
  uint64_t x1;
  fiat_p384_uint1 x2;
  fiat_p384_addcarryx_u64(&x1, &x2, 0x0, (arg2[0]), (arg1[0]));
  uint64_t x3;
  fiat_p384_uint1 x4;
  fiat_p384_addcarryx_u64(&x3, &x4, x2, (arg2[1]), (arg1[1]));
  uint64_t x5;
  fiat_p384_uint1 x6;
  fiat_p384_addcarryx_u64(&x5, &x6, x4, (arg2[2]), (arg1[2]));
  uint64_t x7;
  fiat_p384_uint1 x8;
  fiat_p384_addcarryx_u64(&x7, &x8, x6, (arg2[3]), (arg1[3]));
  uint64_t x9;
  fiat_p384_uint1 x10;
  fiat_p384_addcarryx_u64(&x9, &x10, x8, (arg2[4]), (arg1[4]));
  uint64_t x11;
  fiat_p384_uint1 x12;
  fiat_p384_addcarryx_u64(&x11, &x12, x10, (arg2[5]), (arg1[5]));
  uint64_t x13;
  fiat_p384_uint1 x14;
  fiat_p384_subborrowx_u64(&x13, &x14, 0x0, x1, UINT32_C(0xffffffff));
  uint64_t x15;
  fiat_p384_uint1 x16;
  fiat_p384_subborrowx_u64(&x15, &x16, x14, x3, UINT64_C(0xffffffff00000000));
  uint64_t x17;
  fiat_p384_uint1 x18;
  fiat_p384_subborrowx_u64(&x17, &x18, x16, x5, UINT64_C(0xfffffffffffffffe));
  uint64_t x19;
  fiat_p384_uint1 x20;
  fiat_p384_subborrowx_u64(&x19, &x20, x18, x7, UINT64_C(0xffffffffffffffff));
  uint64_t x21;
  fiat_p384_uint1 x22;
  fiat_p384_subborrowx_u64(&x21, &x22, x20, x9, UINT64_C(0xffffffffffffffff));
  uint64_t x23;
  fiat_p384_uint1 x24;
  fiat_p384_subborrowx_u64(&x23, &x24, x22, x11, UINT64_C(0xffffffffffffffff));
  uint64_t x25;
  fiat_p384_uint1 x26;
  fiat_p384_subborrowx_u64(&x25, &x26, x24, x12, 0x0);
  uint64_t x27;
  fiat_p384_cmovznz_u64(&x27, x26, x13, x1);
  uint64_t x28;
  fiat_p384_cmovznz_u64(&x28, x26, x15, x3);
  uint64_t x29;
  fiat_p384_cmovznz_u64(&x29, x26, x17, x5);
  uint64_t x30;
  fiat_p384_cmovznz_u64(&x30, x26, x19, x7);
  uint64_t x31;
  fiat_p384_cmovznz_u64(&x31, x26, x21, x9);
  uint64_t x32;
  fiat_p384_cmovznz_u64(&x32, x26, x23, x11);
  out1[0] = x27;
  out1[1] = x28;
  out1[2] = x29;
  out1[3] = x30;
  out1[4] = x31;
  out1[5] = x32;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 *   arg2: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_sub(uint64_t out1[6], const uint64_t arg1[6], const uint64_t arg2[6]) {
  uint64_t x1;
  fiat_p384_uint1 x2;
  fiat_p384_subborrowx_u64(&x1, &x2, 0x0, (arg1[0]), (arg2[0]));
  uint64_t x3;
  fiat_p384_uint1 x4;
  fiat_p384_subborrowx_u64(&x3, &x4, x2, (arg1[1]), (arg2[1]));
  uint64_t x5;
  fiat_p384_uint1 x6;
  fiat_p384_subborrowx_u64(&x5, &x6, x4, (arg1[2]), (arg2[2]));
  uint64_t x7;
  fiat_p384_uint1 x8;
  fiat_p384_subborrowx_u64(&x7, &x8, x6, (arg1[3]), (arg2[3]));
  uint64_t x9;
  fiat_p384_uint1 x10;
  fiat_p384_subborrowx_u64(&x9, &x10, x8, (arg1[4]), (arg2[4]));
  uint64_t x11;
  fiat_p384_uint1 x12;
  fiat_p384_subborrowx_u64(&x11, &x12, x10, (arg1[5]), (arg2[5]));
  uint64_t x13;
  fiat_p384_cmovznz_u64(&x13, x12, 0x0, UINT64_C(0xffffffffffffffff));
  uint64_t x14;
  fiat_p384_uint1 x15;
  fiat_p384_addcarryx_u64(&x14, &x15, 0x0, (x13 & UINT32_C(0xffffffff)), x1);
  uint64_t x16;
  fiat_p384_uint1 x17;
  fiat_p384_addcarryx_u64(&x16, &x17, x15, (x13 & UINT64_C(0xffffffff00000000)), x3);
  uint64_t x18;
  fiat_p384_uint1 x19;
  fiat_p384_addcarryx_u64(&x18, &x19, x17, (x13 & UINT64_C(0xfffffffffffffffe)), x5);
  uint64_t x20;
  fiat_p384_uint1 x21;
  fiat_p384_addcarryx_u64(&x20, &x21, x19, (x13 & UINT64_C(0xffffffffffffffff)), x7);
  uint64_t x22;
  fiat_p384_uint1 x23;
  fiat_p384_addcarryx_u64(&x22, &x23, x21, (x13 & UINT64_C(0xffffffffffffffff)), x9);
  uint64_t x24;
  fiat_p384_uint1 x25;
  fiat_p384_addcarryx_u64(&x24, &x25, x23, (x13 & UINT64_C(0xffffffffffffffff)), x11);
  out1[0] = x14;
  out1[1] = x16;
  out1[2] = x18;
  out1[3] = x20;
  out1[4] = x22;
  out1[5] = x24;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_opp(uint64_t out1[6], const uint64_t arg1[6]) {
  uint64_t x1;
  fiat_p384_uint1 x2;
  fiat_p384_subborrowx_u64(&x1, &x2, 0x0, 0x0, (arg1[0]));
  uint64_t x3;
  fiat_p384_uint1 x4;
  fiat_p384_subborrowx_u64(&x3, &x4, x2, 0x0, (arg1[1]));
  uint64_t x5;
  fiat_p384_uint1 x6;
  fiat_p384_subborrowx_u64(&x5, &x6, x4, 0x0, (arg1[2]));
  uint64_t x7;
  fiat_p384_uint1 x8;
  fiat_p384_subborrowx_u64(&x7, &x8, x6, 0x0, (arg1[3]));
  uint64_t x9;
  fiat_p384_uint1 x10;
  fiat_p384_subborrowx_u64(&x9, &x10, x8, 0x0, (arg1[4]));
  uint64_t x11;
  fiat_p384_uint1 x12;
  fiat_p384_subborrowx_u64(&x11, &x12, x10, 0x0, (arg1[5]));
  uint64_t x13;
  fiat_p384_cmovznz_u64(&x13, x12, 0x0, UINT64_C(0xffffffffffffffff));
  uint64_t x14;
  fiat_p384_uint1 x15;
  fiat_p384_addcarryx_u64(&x14, &x15, 0x0, (x13 & UINT32_C(0xffffffff)), x1);
  uint64_t x16;
  fiat_p384_uint1 x17;
  fiat_p384_addcarryx_u64(&x16, &x17, x15, (x13 & UINT64_C(0xffffffff00000000)), x3);
  uint64_t x18;
  fiat_p384_uint1 x19;
  fiat_p384_addcarryx_u64(&x18, &x19, x17, (x13 & UINT64_C(0xfffffffffffffffe)), x5);
  uint64_t x20;
  fiat_p384_uint1 x21;
  fiat_p384_addcarryx_u64(&x20, &x21, x19, (x13 & UINT64_C(0xffffffffffffffff)), x7);
  uint64_t x22;
  fiat_p384_uint1 x23;
  fiat_p384_addcarryx_u64(&x22, &x23, x21, (x13 & UINT64_C(0xffffffffffffffff)), x9);
  uint64_t x24;
  fiat_p384_uint1 x25;
  fiat_p384_addcarryx_u64(&x24, &x25, x23, (x13 & UINT64_C(0xffffffffffffffff)), x11);
  out1[0] = x14;
  out1[1] = x16;
  out1[2] = x18;
  out1[3] = x20;
  out1[4] = x22;
  out1[5] = x24;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_from_montgomery(uint64_t out1[6], const uint64_t arg1[6]) {
  uint64_t x1 = (arg1[0]);
  uint64_t x2;
  uint64_t x3;
  fiat_p384_mulx_u64(&x2, &x3, x1, UINT64_C(0x100000001));
  uint64_t x4;
  uint64_t x5;
  fiat_p384_mulx_u64(&x4, &x5, x2, UINT64_C(0xffffffffffffffff));
  uint64_t x6;
  uint64_t x7;
  fiat_p384_mulx_u64(&x6, &x7, x2, UINT64_C(0xffffffffffffffff));
  uint64_t x8;
  uint64_t x9;
  fiat_p384_mulx_u64(&x8, &x9, x2, UINT64_C(0xffffffffffffffff));
  uint64_t x10;
  uint64_t x11;
  fiat_p384_mulx_u64(&x10, &x11, x2, UINT64_C(0xfffffffffffffffe));
  uint64_t x12;
  uint64_t x13;
  fiat_p384_mulx_u64(&x12, &x13, x2, UINT64_C(0xffffffff00000000));
  uint64_t x14;
  uint64_t x15;
  fiat_p384_mulx_u64(&x14, &x15, x2, UINT32_C(0xffffffff));
  uint64_t x16;
  fiat_p384_uint1 x17;
  fiat_p384_addcarryx_u64(&x16, &x17, 0x0, x12, x15);
  uint64_t x18;
  fiat_p384_uint1 x19;
  fiat_p384_addcarryx_u64(&x18, &x19, x17, x10, x13);
  uint64_t x20;
  fiat_p384_uint1 x21;
  fiat_p384_addcarryx_u64(&x20, &x21, x19, x8, x11);
  uint64_t x22;
  fiat_p384_uint1 x23;
  fiat_p384_addcarryx_u64(&x22, &x23, x21, x6, x9);
  uint64_t x24;
  fiat_p384_uint1 x25;
  fiat_p384_addcarryx_u64(&x24, &x25, x23, x4, x7);
  uint64_t x26;
  fiat_p384_uint1 x27;
  fiat_p384_addcarryx_u64(&x26, &x27, 0x0, x14, x1);
  uint64_t x28;
  fiat_p384_uint1 x29;
  fiat_p384_addcarryx_u64(&x28, &x29, x27, x16, 0x0);
  uint64_t x30;
  fiat_p384_uint1 x31;
  fiat_p384_addcarryx_u64(&x30, &x31, x29, x18, 0x0);
  uint64_t x32;
  fiat_p384_uint1 x33;
  fiat_p384_addcarryx_u64(&x32, &x33, x31, x20, 0x0);
  uint64_t x34;
  fiat_p384_uint1 x35;
  fiat_p384_addcarryx_u64(&x34, &x35, x33, x22, 0x0);
  uint64_t x36;
  fiat_p384_uint1 x37;
  fiat_p384_addcarryx_u64(&x36, &x37, x35, x24, 0x0);
  uint64_t x38;
  fiat_p384_uint1 x39;
  fiat_p384_addcarryx_u64(&x38, &x39, x25, 0x0, x5);
  uint64_t x40;
  fiat_p384_uint1 x41;
  fiat_p384_addcarryx_u64(&x40, &x41, x37, x38, 0x0);
  uint64_t x42;
  fiat_p384_uint1 x43;
  fiat_p384_addcarryx_u64(&x42, &x43, 0x0, (arg1[1]), x28);
  uint64_t x44;
  fiat_p384_uint1 x45;
  fiat_p384_addcarryx_u64(&x44, &x45, x43, 0x0, x30);
  uint64_t x46;
  fiat_p384_uint1 x47;
  fiat_p384_addcarryx_u64(&x46, &x47, x45, 0x0, x32);
  uint64_t x48;
  fiat_p384_uint1 x49;
  fiat_p384_addcarryx_u64(&x48, &x49, x47, 0x0, x34);
  uint64_t x50;
  fiat_p384_uint1 x51;
  fiat_p384_addcarryx_u64(&x50, &x51, x49, 0x0, x36);
  uint64_t x52;
  fiat_p384_uint1 x53;
  fiat_p384_addcarryx_u64(&x52, &x53, x51, 0x0, x40);
  uint64_t x54;
  fiat_p384_uint1 x55;
  fiat_p384_addcarryx_u64(&x54, &x55, x41, 0x0, 0x0);
  uint64_t x56;
  fiat_p384_uint1 x57;
  fiat_p384_addcarryx_u64(&x56, &x57, x53, 0x0, (fiat_p384_uint1)x54);
  uint64_t x58;
  uint64_t x59;
  fiat_p384_mulx_u64(&x58, &x59, x42, UINT64_C(0x100000001));
  uint64_t x60;
  uint64_t x61;
  fiat_p384_mulx_u64(&x60, &x61, x58, UINT64_C(0xffffffffffffffff));
  uint64_t x62;
  uint64_t x63;
  fiat_p384_mulx_u64(&x62, &x63, x58, UINT64_C(0xffffffffffffffff));
  uint64_t x64;
  uint64_t x65;
  fiat_p384_mulx_u64(&x64, &x65, x58, UINT64_C(0xffffffffffffffff));
  uint64_t x66;
  uint64_t x67;
  fiat_p384_mulx_u64(&x66, &x67, x58, UINT64_C(0xfffffffffffffffe));
  uint64_t x68;
  uint64_t x69;
  fiat_p384_mulx_u64(&x68, &x69, x58, UINT64_C(0xffffffff00000000));
  uint64_t x70;
  uint64_t x71;
  fiat_p384_mulx_u64(&x70, &x71, x58, UINT32_C(0xffffffff));
  uint64_t x72;
  fiat_p384_uint1 x73;
  fiat_p384_addcarryx_u64(&x72, &x73, 0x0, x68, x71);
  uint64_t x74;
  fiat_p384_uint1 x75;
  fiat_p384_addcarryx_u64(&x74, &x75, x73, x66, x69);
  uint64_t x76;
  fiat_p384_uint1 x77;
  fiat_p384_addcarryx_u64(&x76, &x77, x75, x64, x67);
  uint64_t x78;
  fiat_p384_uint1 x79;
  fiat_p384_addcarryx_u64(&x78, &x79, x77, x62, x65);
  uint64_t x80;
  fiat_p384_uint1 x81;
  fiat_p384_addcarryx_u64(&x80, &x81, x79, x60, x63);
  uint64_t x82;
  fiat_p384_uint1 x83;
  fiat_p384_addcarryx_u64(&x82, &x83, 0x0, x70, x42);
  uint64_t x84;
  fiat_p384_uint1 x85;
  fiat_p384_addcarryx_u64(&x84, &x85, x83, x72, x44);
  uint64_t x86;
  fiat_p384_uint1 x87;
  fiat_p384_addcarryx_u64(&x86, &x87, x85, x74, x46);
  uint64_t x88;
  fiat_p384_uint1 x89;
  fiat_p384_addcarryx_u64(&x88, &x89, x87, x76, x48);
  uint64_t x90;
  fiat_p384_uint1 x91;
  fiat_p384_addcarryx_u64(&x90, &x91, x89, x78, x50);
  uint64_t x92;
  fiat_p384_uint1 x93;
  fiat_p384_addcarryx_u64(&x92, &x93, x91, x80, x52);
  uint64_t x94;
  fiat_p384_uint1 x95;
  fiat_p384_addcarryx_u64(&x94, &x95, x81, 0x0, x61);
  uint64_t x96;
  fiat_p384_uint1 x97;
  fiat_p384_addcarryx_u64(&x96, &x97, x93, x94, x56);
  uint64_t x98;
  fiat_p384_uint1 x99;
  fiat_p384_addcarryx_u64(&x98, &x99, 0x0, (arg1[2]), x84);
  uint64_t x100;
  fiat_p384_uint1 x101;
  fiat_p384_addcarryx_u64(&x100, &x101, x99, 0x0, x86);
  uint64_t x102;
  fiat_p384_uint1 x103;
  fiat_p384_addcarryx_u64(&x102, &x103, x101, 0x0, x88);
  uint64_t x104;
  fiat_p384_uint1 x105;
  fiat_p384_addcarryx_u64(&x104, &x105, x103, 0x0, x90);
  uint64_t x106;
  fiat_p384_uint1 x107;
  fiat_p384_addcarryx_u64(&x106, &x107, x105, 0x0, x92);
  uint64_t x108;
  fiat_p384_uint1 x109;
  fiat_p384_addcarryx_u64(&x108, &x109, x107, 0x0, x96);
  uint64_t x110;
  fiat_p384_uint1 x111;
  fiat_p384_addcarryx_u64(&x110, &x111, x97, 0x0, x57);
  uint64_t x112;
  fiat_p384_uint1 x113;
  fiat_p384_addcarryx_u64(&x112, &x113, x109, 0x0, (fiat_p384_uint1)x110);
  uint64_t x114;
  uint64_t x115;
  fiat_p384_mulx_u64(&x114, &x115, x98, UINT64_C(0x100000001));
  uint64_t x116;
  uint64_t x117;
  fiat_p384_mulx_u64(&x116, &x117, x114, UINT64_C(0xffffffffffffffff));
  uint64_t x118;
  uint64_t x119;
  fiat_p384_mulx_u64(&x118, &x119, x114, UINT64_C(0xffffffffffffffff));
  uint64_t x120;
  uint64_t x121;
  fiat_p384_mulx_u64(&x120, &x121, x114, UINT64_C(0xffffffffffffffff));
  uint64_t x122;
  uint64_t x123;
  fiat_p384_mulx_u64(&x122, &x123, x114, UINT64_C(0xfffffffffffffffe));
  uint64_t x124;
  uint64_t x125;
  fiat_p384_mulx_u64(&x124, &x125, x114, UINT64_C(0xffffffff00000000));
  uint64_t x126;
  uint64_t x127;
  fiat_p384_mulx_u64(&x126, &x127, x114, UINT32_C(0xffffffff));
  uint64_t x128;
  fiat_p384_uint1 x129;
  fiat_p384_addcarryx_u64(&x128, &x129, 0x0, x124, x127);
  uint64_t x130;
  fiat_p384_uint1 x131;
  fiat_p384_addcarryx_u64(&x130, &x131, x129, x122, x125);
  uint64_t x132;
  fiat_p384_uint1 x133;
  fiat_p384_addcarryx_u64(&x132, &x133, x131, x120, x123);
  uint64_t x134;
  fiat_p384_uint1 x135;
  fiat_p384_addcarryx_u64(&x134, &x135, x133, x118, x121);
  uint64_t x136;
  fiat_p384_uint1 x137;
  fiat_p384_addcarryx_u64(&x136, &x137, x135, x116, x119);
  uint64_t x138;
  fiat_p384_uint1 x139;
  fiat_p384_addcarryx_u64(&x138, &x139, 0x0, x126, x98);
  uint64_t x140;
  fiat_p384_uint1 x141;
  fiat_p384_addcarryx_u64(&x140, &x141, x139, x128, x100);
  uint64_t x142;
  fiat_p384_uint1 x143;
  fiat_p384_addcarryx_u64(&x142, &x143, x141, x130, x102);
  uint64_t x144;
  fiat_p384_uint1 x145;
  fiat_p384_addcarryx_u64(&x144, &x145, x143, x132, x104);
  uint64_t x146;
  fiat_p384_uint1 x147;
  fiat_p384_addcarryx_u64(&x146, &x147, x145, x134, x106);
  uint64_t x148;
  fiat_p384_uint1 x149;
  fiat_p384_addcarryx_u64(&x148, &x149, x147, x136, x108);
  uint64_t x150;
  fiat_p384_uint1 x151;
  fiat_p384_addcarryx_u64(&x150, &x151, x137, 0x0, x117);
  uint64_t x152;
  fiat_p384_uint1 x153;
  fiat_p384_addcarryx_u64(&x152, &x153, x149, x150, x112);
  uint64_t x154;
  fiat_p384_uint1 x155;
  fiat_p384_addcarryx_u64(&x154, &x155, 0x0, (arg1[3]), x140);
  uint64_t x156;
  fiat_p384_uint1 x157;
  fiat_p384_addcarryx_u64(&x156, &x157, x155, 0x0, x142);
  uint64_t x158;
  fiat_p384_uint1 x159;
  fiat_p384_addcarryx_u64(&x158, &x159, x157, 0x0, x144);
  uint64_t x160;
  fiat_p384_uint1 x161;
  fiat_p384_addcarryx_u64(&x160, &x161, x159, 0x0, x146);
  uint64_t x162;
  fiat_p384_uint1 x163;
  fiat_p384_addcarryx_u64(&x162, &x163, x161, 0x0, x148);
  uint64_t x164;
  fiat_p384_uint1 x165;
  fiat_p384_addcarryx_u64(&x164, &x165, x163, 0x0, x152);
  uint64_t x166;
  fiat_p384_uint1 x167;
  fiat_p384_addcarryx_u64(&x166, &x167, x153, 0x0, x113);
  uint64_t x168;
  fiat_p384_uint1 x169;
  fiat_p384_addcarryx_u64(&x168, &x169, x165, 0x0, (fiat_p384_uint1)x166);
  uint64_t x170;
  uint64_t x171;
  fiat_p384_mulx_u64(&x170, &x171, x154, UINT64_C(0x100000001));
  uint64_t x172;
  uint64_t x173;
  fiat_p384_mulx_u64(&x172, &x173, x170, UINT64_C(0xffffffffffffffff));
  uint64_t x174;
  uint64_t x175;
  fiat_p384_mulx_u64(&x174, &x175, x170, UINT64_C(0xffffffffffffffff));
  uint64_t x176;
  uint64_t x177;
  fiat_p384_mulx_u64(&x176, &x177, x170, UINT64_C(0xffffffffffffffff));
  uint64_t x178;
  uint64_t x179;
  fiat_p384_mulx_u64(&x178, &x179, x170, UINT64_C(0xfffffffffffffffe));
  uint64_t x180;
  uint64_t x181;
  fiat_p384_mulx_u64(&x180, &x181, x170, UINT64_C(0xffffffff00000000));
  uint64_t x182;
  uint64_t x183;
  fiat_p384_mulx_u64(&x182, &x183, x170, UINT32_C(0xffffffff));
  uint64_t x184;
  fiat_p384_uint1 x185;
  fiat_p384_addcarryx_u64(&x184, &x185, 0x0, x180, x183);
  uint64_t x186;
  fiat_p384_uint1 x187;
  fiat_p384_addcarryx_u64(&x186, &x187, x185, x178, x181);
  uint64_t x188;
  fiat_p384_uint1 x189;
  fiat_p384_addcarryx_u64(&x188, &x189, x187, x176, x179);
  uint64_t x190;
  fiat_p384_uint1 x191;
  fiat_p384_addcarryx_u64(&x190, &x191, x189, x174, x177);
  uint64_t x192;
  fiat_p384_uint1 x193;
  fiat_p384_addcarryx_u64(&x192, &x193, x191, x172, x175);
  uint64_t x194;
  fiat_p384_uint1 x195;
  fiat_p384_addcarryx_u64(&x194, &x195, 0x0, x182, x154);
  uint64_t x196;
  fiat_p384_uint1 x197;
  fiat_p384_addcarryx_u64(&x196, &x197, x195, x184, x156);
  uint64_t x198;
  fiat_p384_uint1 x199;
  fiat_p384_addcarryx_u64(&x198, &x199, x197, x186, x158);
  uint64_t x200;
  fiat_p384_uint1 x201;
  fiat_p384_addcarryx_u64(&x200, &x201, x199, x188, x160);
  uint64_t x202;
  fiat_p384_uint1 x203;
  fiat_p384_addcarryx_u64(&x202, &x203, x201, x190, x162);
  uint64_t x204;
  fiat_p384_uint1 x205;
  fiat_p384_addcarryx_u64(&x204, &x205, x203, x192, x164);
  uint64_t x206;
  fiat_p384_uint1 x207;
  fiat_p384_addcarryx_u64(&x206, &x207, x193, 0x0, x173);
  uint64_t x208;
  fiat_p384_uint1 x209;
  fiat_p384_addcarryx_u64(&x208, &x209, x205, x206, x168);
  uint64_t x210;
  fiat_p384_uint1 x211;
  fiat_p384_addcarryx_u64(&x210, &x211, 0x0, (arg1[4]), x196);
  uint64_t x212;
  fiat_p384_uint1 x213;
  fiat_p384_addcarryx_u64(&x212, &x213, x211, 0x0, x198);
  uint64_t x214;
  fiat_p384_uint1 x215;
  fiat_p384_addcarryx_u64(&x214, &x215, x213, 0x0, x200);
  uint64_t x216;
  fiat_p384_uint1 x217;
  fiat_p384_addcarryx_u64(&x216, &x217, x215, 0x0, x202);
  uint64_t x218;
  fiat_p384_uint1 x219;
  fiat_p384_addcarryx_u64(&x218, &x219, x217, 0x0, x204);
  uint64_t x220;
  fiat_p384_uint1 x221;
  fiat_p384_addcarryx_u64(&x220, &x221, x219, 0x0, x208);
  uint64_t x222;
  fiat_p384_uint1 x223;
  fiat_p384_addcarryx_u64(&x222, &x223, x209, 0x0, x169);
  uint64_t x224;
  fiat_p384_uint1 x225;
  fiat_p384_addcarryx_u64(&x224, &x225, x221, 0x0, (fiat_p384_uint1)x222);
  uint64_t x226;
  uint64_t x227;
  fiat_p384_mulx_u64(&x226, &x227, x210, UINT64_C(0x100000001));
  uint64_t x228;
  uint64_t x229;
  fiat_p384_mulx_u64(&x228, &x229, x226, UINT64_C(0xffffffffffffffff));
  uint64_t x230;
  uint64_t x231;
  fiat_p384_mulx_u64(&x230, &x231, x226, UINT64_C(0xffffffffffffffff));
  uint64_t x232;
  uint64_t x233;
  fiat_p384_mulx_u64(&x232, &x233, x226, UINT64_C(0xffffffffffffffff));
  uint64_t x234;
  uint64_t x235;
  fiat_p384_mulx_u64(&x234, &x235, x226, UINT64_C(0xfffffffffffffffe));
  uint64_t x236;
  uint64_t x237;
  fiat_p384_mulx_u64(&x236, &x237, x226, UINT64_C(0xffffffff00000000));
  uint64_t x238;
  uint64_t x239;
  fiat_p384_mulx_u64(&x238, &x239, x226, UINT32_C(0xffffffff));
  uint64_t x240;
  fiat_p384_uint1 x241;
  fiat_p384_addcarryx_u64(&x240, &x241, 0x0, x236, x239);
  uint64_t x242;
  fiat_p384_uint1 x243;
  fiat_p384_addcarryx_u64(&x242, &x243, x241, x234, x237);
  uint64_t x244;
  fiat_p384_uint1 x245;
  fiat_p384_addcarryx_u64(&x244, &x245, x243, x232, x235);
  uint64_t x246;
  fiat_p384_uint1 x247;
  fiat_p384_addcarryx_u64(&x246, &x247, x245, x230, x233);
  uint64_t x248;
  fiat_p384_uint1 x249;
  fiat_p384_addcarryx_u64(&x248, &x249, x247, x228, x231);
  uint64_t x250;
  fiat_p384_uint1 x251;
  fiat_p384_addcarryx_u64(&x250, &x251, 0x0, x238, x210);
  uint64_t x252;
  fiat_p384_uint1 x253;
  fiat_p384_addcarryx_u64(&x252, &x253, x251, x240, x212);
  uint64_t x254;
  fiat_p384_uint1 x255;
  fiat_p384_addcarryx_u64(&x254, &x255, x253, x242, x214);
  uint64_t x256;
  fiat_p384_uint1 x257;
  fiat_p384_addcarryx_u64(&x256, &x257, x255, x244, x216);
  uint64_t x258;
  fiat_p384_uint1 x259;
  fiat_p384_addcarryx_u64(&x258, &x259, x257, x246, x218);
  uint64_t x260;
  fiat_p384_uint1 x261;
  fiat_p384_addcarryx_u64(&x260, &x261, x259, x248, x220);
  uint64_t x262;
  fiat_p384_uint1 x263;
  fiat_p384_addcarryx_u64(&x262, &x263, x249, 0x0, x229);
  uint64_t x264;
  fiat_p384_uint1 x265;
  fiat_p384_addcarryx_u64(&x264, &x265, x261, x262, x224);
  uint64_t x266;
  fiat_p384_uint1 x267;
  fiat_p384_addcarryx_u64(&x266, &x267, 0x0, (arg1[5]), x252);
  uint64_t x268;
  fiat_p384_uint1 x269;
  fiat_p384_addcarryx_u64(&x268, &x269, x267, 0x0, x254);
  uint64_t x270;
  fiat_p384_uint1 x271;
  fiat_p384_addcarryx_u64(&x270, &x271, x269, 0x0, x256);
  uint64_t x272;
  fiat_p384_uint1 x273;
  fiat_p384_addcarryx_u64(&x272, &x273, x271, 0x0, x258);
  uint64_t x274;
  fiat_p384_uint1 x275;
  fiat_p384_addcarryx_u64(&x274, &x275, x273, 0x0, x260);
  uint64_t x276;
  fiat_p384_uint1 x277;
  fiat_p384_addcarryx_u64(&x276, &x277, x275, 0x0, x264);
  uint64_t x278;
  fiat_p384_uint1 x279;
  fiat_p384_addcarryx_u64(&x278, &x279, x265, 0x0, x225);
  uint64_t x280;
  fiat_p384_uint1 x281;
  fiat_p384_addcarryx_u64(&x280, &x281, x277, 0x0, (fiat_p384_uint1)x278);
  uint64_t x282;
  uint64_t x283;
  fiat_p384_mulx_u64(&x282, &x283, x266, UINT64_C(0x100000001));
  uint64_t x284;
  uint64_t x285;
  fiat_p384_mulx_u64(&x284, &x285, x282, UINT64_C(0xffffffffffffffff));
  uint64_t x286;
  uint64_t x287;
  fiat_p384_mulx_u64(&x286, &x287, x282, UINT64_C(0xffffffffffffffff));
  uint64_t x288;
  uint64_t x289;
  fiat_p384_mulx_u64(&x288, &x289, x282, UINT64_C(0xffffffffffffffff));
  uint64_t x290;
  uint64_t x291;
  fiat_p384_mulx_u64(&x290, &x291, x282, UINT64_C(0xfffffffffffffffe));
  uint64_t x292;
  uint64_t x293;
  fiat_p384_mulx_u64(&x292, &x293, x282, UINT64_C(0xffffffff00000000));
  uint64_t x294;
  uint64_t x295;
  fiat_p384_mulx_u64(&x294, &x295, x282, UINT32_C(0xffffffff));
  uint64_t x296;
  fiat_p384_uint1 x297;
  fiat_p384_addcarryx_u64(&x296, &x297, 0x0, x292, x295);
  uint64_t x298;
  fiat_p384_uint1 x299;
  fiat_p384_addcarryx_u64(&x298, &x299, x297, x290, x293);
  uint64_t x300;
  fiat_p384_uint1 x301;
  fiat_p384_addcarryx_u64(&x300, &x301, x299, x288, x291);
  uint64_t x302;
  fiat_p384_uint1 x303;
  fiat_p384_addcarryx_u64(&x302, &x303, x301, x286, x289);
  uint64_t x304;
  fiat_p384_uint1 x305;
  fiat_p384_addcarryx_u64(&x304, &x305, x303, x284, x287);
  uint64_t x306;
  fiat_p384_uint1 x307;
  fiat_p384_addcarryx_u64(&x306, &x307, 0x0, x294, x266);
  uint64_t x308;
  fiat_p384_uint1 x309;
  fiat_p384_addcarryx_u64(&x308, &x309, x307, x296, x268);
  uint64_t x310;
  fiat_p384_uint1 x311;
  fiat_p384_addcarryx_u64(&x310, &x311, x309, x298, x270);
  uint64_t x312;
  fiat_p384_uint1 x313;
  fiat_p384_addcarryx_u64(&x312, &x313, x311, x300, x272);
  uint64_t x314;
  fiat_p384_uint1 x315;
  fiat_p384_addcarryx_u64(&x314, &x315, x313, x302, x274);
  uint64_t x316;
  fiat_p384_uint1 x317;
  fiat_p384_addcarryx_u64(&x316, &x317, x315, x304, x276);
  uint64_t x318;
  fiat_p384_uint1 x319;
  fiat_p384_addcarryx_u64(&x318, &x319, x305, 0x0, x285);
  uint64_t x320;
  fiat_p384_uint1 x321;
  fiat_p384_addcarryx_u64(&x320, &x321, x317, x318, x280);
  uint64_t x322;
  fiat_p384_uint1 x323;
  fiat_p384_subborrowx_u64(&x322, &x323, 0x0, x308, UINT32_C(0xffffffff));
  uint64_t x324;
  fiat_p384_uint1 x325;
  fiat_p384_subborrowx_u64(&x324, &x325, x323, x310, UINT64_C(0xffffffff00000000));
  uint64_t x326;
  fiat_p384_uint1 x327;
  fiat_p384_subborrowx_u64(&x326, &x327, x325, x312, UINT64_C(0xfffffffffffffffe));
  uint64_t x328;
  fiat_p384_uint1 x329;
  fiat_p384_subborrowx_u64(&x328, &x329, x327, x314, UINT64_C(0xffffffffffffffff));
  uint64_t x330;
  fiat_p384_uint1 x331;
  fiat_p384_subborrowx_u64(&x330, &x331, x329, x316, UINT64_C(0xffffffffffffffff));
  uint64_t x332;
  fiat_p384_uint1 x333;
  fiat_p384_subborrowx_u64(&x332, &x333, x331, x320, UINT64_C(0xffffffffffffffff));
  uint64_t x334;
  fiat_p384_uint1 x335;
  fiat_p384_addcarryx_u64(&x334, &x335, x321, 0x0, x281);
  uint64_t x336;
  fiat_p384_uint1 x337;
  fiat_p384_subborrowx_u64(&x336, &x337, x333, (fiat_p384_uint1)x334, 0x0);
  uint64_t x338;
  fiat_p384_cmovznz_u64(&x338, x337, x322, x308);
  uint64_t x339;
  fiat_p384_cmovznz_u64(&x339, x337, x324, x310);
  uint64_t x340;
  fiat_p384_cmovznz_u64(&x340, x337, x326, x312);
  uint64_t x341;
  fiat_p384_cmovznz_u64(&x341, x337, x328, x314);
  uint64_t x342;
  fiat_p384_cmovznz_u64(&x342, x337, x330, x316);
  uint64_t x343;
  fiat_p384_cmovznz_u64(&x343, x337, x332, x320);
  out1[0] = x338;
  out1[1] = x339;
  out1[2] = x340;
  out1[3] = x341;
  out1[4] = x342;
  out1[5] = x343;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [0x0 ~> 0xffffffffffffffff]
 */
static void fiat_p384_nonzero(uint64_t* out1, const uint64_t arg1[6]) {
  uint64_t x1 = ((arg1[0]) | ((arg1[1]) | ((arg1[2]) | ((arg1[3]) | ((arg1[4]) | ((arg1[5]) | (uint64_t)0x0))))));
  *out1 = x1;
}

/*
 * Input Bounds:
 *   arg1: [0x0 ~> 0x1]
 *   arg2: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 *   arg3: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_selectznz(uint64_t out1[6], fiat_p384_uint1 arg1, const uint64_t arg2[6], const uint64_t arg3[6]) {
  uint64_t x1;
  fiat_p384_cmovznz_u64(&x1, arg1, (arg2[0]), (arg3[0]));
  uint64_t x2;
  fiat_p384_cmovznz_u64(&x2, arg1, (arg2[1]), (arg3[1]));
  uint64_t x3;
  fiat_p384_cmovznz_u64(&x3, arg1, (arg2[2]), (arg3[2]));
  uint64_t x4;
  fiat_p384_cmovznz_u64(&x4, arg1, (arg2[3]), (arg3[3]));
  uint64_t x5;
  fiat_p384_cmovznz_u64(&x5, arg1, (arg2[4]), (arg3[4]));
  uint64_t x6;
  fiat_p384_cmovznz_u64(&x6, arg1, (arg2[5]), (arg3[5]));
  out1[0] = x1;
  out1[1] = x2;
  out1[2] = x3;
  out1[3] = x4;
  out1[4] = x5;
  out1[5] = x6;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
 */
static void fiat_p384_to_bytes(uint8_t out1[48], const uint64_t arg1[6]) {
  uint64_t x1 = (arg1[5]);
  uint64_t x2 = (arg1[4]);
  uint64_t x3 = (arg1[3]);
  uint64_t x4 = (arg1[2]);
  uint64_t x5 = (arg1[1]);
  uint64_t x6 = (arg1[0]);
  uint64_t x7 = (x6 >> 8);
  uint8_t x8 = (uint8_t)(x6 & UINT8_C(0xff));
  uint64_t x9 = (x7 >> 8);
  uint8_t x10 = (uint8_t)(x7 & UINT8_C(0xff));
  uint64_t x11 = (x9 >> 8);
  uint8_t x12 = (uint8_t)(x9 & UINT8_C(0xff));
  uint64_t x13 = (x11 >> 8);
  uint8_t x14 = (uint8_t)(x11 & UINT8_C(0xff));
  uint64_t x15 = (x13 >> 8);
  uint8_t x16 = (uint8_t)(x13 & UINT8_C(0xff));
  uint64_t x17 = (x15 >> 8);
  uint8_t x18 = (uint8_t)(x15 & UINT8_C(0xff));
  uint8_t x19 = (uint8_t)(x17 >> 8);
  uint8_t x20 = (uint8_t)(x17 & UINT8_C(0xff));
  fiat_p384_uint1 x21 = (fiat_p384_uint1)(x19 >> 8);
  uint8_t x22 = (uint8_t)(x19 & UINT8_C(0xff));
  uint64_t x23 = (x21 + x5);
  uint64_t x24 = (x23 >> 8);
  uint8_t x25 = (uint8_t)(x23 & UINT8_C(0xff));
  uint64_t x26 = (x24 >> 8);
  uint8_t x27 = (uint8_t)(x24 & UINT8_C(0xff));
  uint64_t x28 = (x26 >> 8);
  uint8_t x29 = (uint8_t)(x26 & UINT8_C(0xff));
  uint64_t x30 = (x28 >> 8);
  uint8_t x31 = (uint8_t)(x28 & UINT8_C(0xff));
  uint64_t x32 = (x30 >> 8);
  uint8_t x33 = (uint8_t)(x30 & UINT8_C(0xff));
  uint64_t x34 = (x32 >> 8);
  uint8_t x35 = (uint8_t)(x32 & UINT8_C(0xff));
  uint8_t x36 = (uint8_t)(x34 >> 8);
  uint8_t x37 = (uint8_t)(x34 & UINT8_C(0xff));
  fiat_p384_uint1 x38 = (fiat_p384_uint1)(x36 >> 8);
  uint8_t x39 = (uint8_t)(x36 & UINT8_C(0xff));
  uint64_t x40 = (x38 + x4);
  uint64_t x41 = (x40 >> 8);
  uint8_t x42 = (uint8_t)(x40 & UINT8_C(0xff));
  uint64_t x43 = (x41 >> 8);
  uint8_t x44 = (uint8_t)(x41 & UINT8_C(0xff));
  uint64_t x45 = (x43 >> 8);
  uint8_t x46 = (uint8_t)(x43 & UINT8_C(0xff));
  uint64_t x47 = (x45 >> 8);
  uint8_t x48 = (uint8_t)(x45 & UINT8_C(0xff));
  uint64_t x49 = (x47 >> 8);
  uint8_t x50 = (uint8_t)(x47 & UINT8_C(0xff));
  uint64_t x51 = (x49 >> 8);
  uint8_t x52 = (uint8_t)(x49 & UINT8_C(0xff));
  uint8_t x53 = (uint8_t)(x51 >> 8);
  uint8_t x54 = (uint8_t)(x51 & UINT8_C(0xff));
  fiat_p384_uint1 x55 = (fiat_p384_uint1)(x53 >> 8);
  uint8_t x56 = (uint8_t)(x53 & UINT8_C(0xff));
  uint64_t x57 = (x55 + x3);
  uint64_t x58 = (x57 >> 8);
  uint8_t x59 = (uint8_t)(x57 & UINT8_C(0xff));
  uint64_t x60 = (x58 >> 8);
  uint8_t x61 = (uint8_t)(x58 & UINT8_C(0xff));
  uint64_t x62 = (x60 >> 8);
  uint8_t x63 = (uint8_t)(x60 & UINT8_C(0xff));
  uint64_t x64 = (x62 >> 8);
  uint8_t x65 = (uint8_t)(x62 & UINT8_C(0xff));
  uint64_t x66 = (x64 >> 8);
  uint8_t x67 = (uint8_t)(x64 & UINT8_C(0xff));
  uint64_t x68 = (x66 >> 8);
  uint8_t x69 = (uint8_t)(x66 & UINT8_C(0xff));
  uint8_t x70 = (uint8_t)(x68 >> 8);
  uint8_t x71 = (uint8_t)(x68 & UINT8_C(0xff));
  fiat_p384_uint1 x72 = (fiat_p384_uint1)(x70 >> 8);
  uint8_t x73 = (uint8_t)(x70 & UINT8_C(0xff));
  uint64_t x74 = (x72 + x2);
  uint64_t x75 = (x74 >> 8);
  uint8_t x76 = (uint8_t)(x74 & UINT8_C(0xff));
  uint64_t x77 = (x75 >> 8);
  uint8_t x78 = (uint8_t)(x75 & UINT8_C(0xff));
  uint64_t x79 = (x77 >> 8);
  uint8_t x80 = (uint8_t)(x77 & UINT8_C(0xff));
  uint64_t x81 = (x79 >> 8);
  uint8_t x82 = (uint8_t)(x79 & UINT8_C(0xff));
  uint64_t x83 = (x81 >> 8);
  uint8_t x84 = (uint8_t)(x81 & UINT8_C(0xff));
  uint64_t x85 = (x83 >> 8);
  uint8_t x86 = (uint8_t)(x83 & UINT8_C(0xff));
  uint8_t x87 = (uint8_t)(x85 >> 8);
  uint8_t x88 = (uint8_t)(x85 & UINT8_C(0xff));
  fiat_p384_uint1 x89 = (fiat_p384_uint1)(x87 >> 8);
  uint8_t x90 = (uint8_t)(x87 & UINT8_C(0xff));
  uint64_t x91 = (x89 + x1);
  uint64_t x92 = (x91 >> 8);
  uint8_t x93 = (uint8_t)(x91 & UINT8_C(0xff));
  uint64_t x94 = (x92 >> 8);
  uint8_t x95 = (uint8_t)(x92 & UINT8_C(0xff));
  uint64_t x96 = (x94 >> 8);
  uint8_t x97 = (uint8_t)(x94 & UINT8_C(0xff));
  uint64_t x98 = (x96 >> 8);
  uint8_t x99 = (uint8_t)(x96 & UINT8_C(0xff));
  uint64_t x100 = (x98 >> 8);
  uint8_t x101 = (uint8_t)(x98 & UINT8_C(0xff));
  uint64_t x102 = (x100 >> 8);
  uint8_t x103 = (uint8_t)(x100 & UINT8_C(0xff));
  uint8_t x104 = (uint8_t)(x102 >> 8);
  uint8_t x105 = (uint8_t)(x102 & UINT8_C(0xff));
  out1[0] = x8;
  out1[1] = x10;
  out1[2] = x12;
  out1[3] = x14;
  out1[4] = x16;
  out1[5] = x18;
  out1[6] = x20;
  out1[7] = x22;
  out1[8] = x25;
  out1[9] = x27;
  out1[10] = x29;
  out1[11] = x31;
  out1[12] = x33;
  out1[13] = x35;
  out1[14] = x37;
  out1[15] = x39;
  out1[16] = x42;
  out1[17] = x44;
  out1[18] = x46;
  out1[19] = x48;
  out1[20] = x50;
  out1[21] = x52;
  out1[22] = x54;
  out1[23] = x56;
  out1[24] = x59;
  out1[25] = x61;
  out1[26] = x63;
  out1[27] = x65;
  out1[28] = x67;
  out1[29] = x69;
  out1[30] = x71;
  out1[31] = x73;
  out1[32] = x76;
  out1[33] = x78;
  out1[34] = x80;
  out1[35] = x82;
  out1[36] = x84;
  out1[37] = x86;
  out1[38] = x88;
  out1[39] = x90;
  out1[40] = x93;
  out1[41] = x95;
  out1[42] = x97;
  out1[43] = x99;
  out1[44] = x101;
  out1[45] = x103;
  out1[46] = x105;
  out1[47] = x104;
}

/*
 * Input Bounds:
 *   arg1: [[0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff], [0x0 ~> 0xff]]
 * Output Bounds:
 *   out1: [[0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff], [0x0 ~> 0xffffffffffffffff]]
 */
static void fiat_p384_from_bytes(uint64_t out1[6], const uint8_t arg1[48]) {
  uint64_t x1 = ((uint64_t)(arg1[47]) << 56);
  uint64_t x2 = ((uint64_t)(arg1[46]) << 48);
  uint64_t x3 = ((uint64_t)(arg1[45]) << 40);
  uint64_t x4 = ((uint64_t)(arg1[44]) << 32);
  uint64_t x5 = ((uint64_t)(arg1[43]) << 24);
  uint64_t x6 = ((uint64_t)(arg1[42]) << 16);
  uint64_t x7 = ((uint64_t)(arg1[41]) << 8);
  uint8_t x8 = (arg1[40]);
  uint64_t x9 = ((uint64_t)(arg1[39]) << 56);
  uint64_t x10 = ((uint64_t)(arg1[38]) << 48);
  uint64_t x11 = ((uint64_t)(arg1[37]) << 40);
  uint64_t x12 = ((uint64_t)(arg1[36]) << 32);
  uint64_t x13 = ((uint64_t)(arg1[35]) << 24);
  uint64_t x14 = ((uint64_t)(arg1[34]) << 16);
  uint64_t x15 = ((uint64_t)(arg1[33]) << 8);
  uint8_t x16 = (arg1[32]);
  uint64_t x17 = ((uint64_t)(arg1[31]) << 56);
  uint64_t x18 = ((uint64_t)(arg1[30]) << 48);
  uint64_t x19 = ((uint64_t)(arg1[29]) << 40);
  uint64_t x20 = ((uint64_t)(arg1[28]) << 32);
  uint64_t x21 = ((uint64_t)(arg1[27]) << 24);
  uint64_t x22 = ((uint64_t)(arg1[26]) << 16);
  uint64_t x23 = ((uint64_t)(arg1[25]) << 8);
  uint8_t x24 = (arg1[24]);
  uint64_t x25 = ((uint64_t)(arg1[23]) << 56);
  uint64_t x26 = ((uint64_t)(arg1[22]) << 48);
  uint64_t x27 = ((uint64_t)(arg1[21]) << 40);
  uint64_t x28 = ((uint64_t)(arg1[20]) << 32);
  uint64_t x29 = ((uint64_t)(arg1[19]) << 24);
  uint64_t x30 = ((uint64_t)(arg1[18]) << 16);
  uint64_t x31 = ((uint64_t)(arg1[17]) << 8);
  uint8_t x32 = (arg1[16]);
  uint64_t x33 = ((uint64_t)(arg1[15]) << 56);
  uint64_t x34 = ((uint64_t)(arg1[14]) << 48);
  uint64_t x35 = ((uint64_t)(arg1[13]) << 40);
  uint64_t x36 = ((uint64_t)(arg1[12]) << 32);
  uint64_t x37 = ((uint64_t)(arg1[11]) << 24);
  uint64_t x38 = ((uint64_t)(arg1[10]) << 16);
  uint64_t x39 = ((uint64_t)(arg1[9]) << 8);
  uint8_t x40 = (arg1[8]);
  uint64_t x41 = ((uint64_t)(arg1[7]) << 56);
  uint64_t x42 = ((uint64_t)(arg1[6]) << 48);
  uint64_t x43 = ((uint64_t)(arg1[5]) << 40);
  uint64_t x44 = ((uint64_t)(arg1[4]) << 32);
  uint64_t x45 = ((uint64_t)(arg1[3]) << 24);
  uint64_t x46 = ((uint64_t)(arg1[2]) << 16);
  uint64_t x47 = ((uint64_t)(arg1[1]) << 8);
  uint8_t x48 = (arg1[0]);
  uint64_t x49 = (x48 + (x47 + (x46 + (x45 + (x44 + (x43 + (x42 + x41)))))));
  fiat_p384_uint1 x50 = (fiat_p384_uint1)((fiat_p384_uint128)x49 >> 64);
  uint64_t x51 = (x49 & UINT64_C(0xffffffffffffffff));
  uint64_t x52 = (x8 + (x7 + (x6 + (x5 + (x4 + (x3 + (x2 + x1)))))));
  uint64_t x53 = (x16 + (x15 + (x14 + (x13 + (x12 + (x11 + (x10 + x9)))))));
  uint64_t x54 = (x24 + (x23 + (x22 + (x21 + (x20 + (x19 + (x18 + x17)))))));
  uint64_t x55 = (x32 + (x31 + (x30 + (x29 + (x28 + (x27 + (x26 + x25)))))));
  uint64_t x56 = (x40 + (x39 + (x38 + (x37 + (x36 + (x35 + (x34 + x33)))))));
  uint64_t x57 = (x50 + x56);
  fiat_p384_uint1 x58 = (fiat_p384_uint1)((fiat_p384_uint128)x57 >> 64);
  uint64_t x59 = (x57 & UINT64_C(0xffffffffffffffff));
  uint64_t x60 = (x58 + x55);
  fiat_p384_uint1 x61 = (fiat_p384_uint1)((fiat_p384_uint128)x60 >> 64);
  uint64_t x62 = (x60 & UINT64_C(0xffffffffffffffff));
  uint64_t x63 = (x61 + x54);
  fiat_p384_uint1 x64 = (fiat_p384_uint1)((fiat_p384_uint128)x63 >> 64);
  uint64_t x65 = (x63 & UINT64_C(0xffffffffffffffff));
  uint64_t x66 = (x64 + x53);
  fiat_p384_uint1 x67 = (fiat_p384_uint1)((fiat_p384_uint128)x66 >> 64);
  uint64_t x68 = (x66 & UINT64_C(0xffffffffffffffff));
  uint64_t x69 = (x67 + x52);
  out1[0] = x51;
  out1[1] = x59;
  out1[2] = x62;
  out1[3] = x65;
  out1[4] = x68;
  out1[5] = x69;
}

